@@ -39,6 +39,22 @@ extern int precache_model_skin;
 
 extern byte *precache_model;
 
+// Forces all downloads to UDP.
+qboolean forceudp = false;
+
+/* This - and some more code downn below - are the 'Crazy Fallback
+   Magic'. First we're trying to download all files over HTTP with
+   r1q2-style URLs. If we encountered errors we reset the complete
+   precacher state and retry with HTTP and q2pro-style URLs. If we
+   still got errors we're falling back to UDP. So:
+     - 0: Virgin state, r1q2-style URLs.
+     - 1: Second iteration, q2pro-style URL.
+     - 3: Third iteration, UDP downloads. */
+static unsigned int precacherIteration;
+
+// r1q2 searches the global filelist at /, q2pro at /gamedir...
+static qboolean gamedirForFilelist = false;
+
 static const char *env_suf[6] = {"rt", "bk", "lf", "ft", "up", "dn"};
 
 #define PLAYER_MULT 5
@@ -54,6 +70,43 @@ CL_RequestNextDownload(void)
 	char fn[MAX_OSPATH];
 	dmdl_t *pheader;
 
+	if (precacherIteration == 0)
+	{
+#if USE_CURL
+		// r1q2-style URLs.
+		CL_HTTP_SetDownloadGamedir(cl.gamedir);
+#endif
+	}
+	else if (precacherIteration == 1)
+	{
+#if USE_CURL
+		// q2pro-style URLs.
+		if (cl.gamedir[0] == '\0')
+		{
+			CL_HTTP_SetDownloadGamedir(BASEDIRNAME);
+		}
+		else
+		{
+			CL_HTTP_SetDownloadGamedir(cl.gamedir);
+		}
+
+		// Force another try with the filelist.
+		CL_HTTP_EnableGenericFilelist();
+		gamedirForFilelist = true;
+#endif
+	}
+	else if (precacherIteration == 2)
+	{
+		// UDP Fallback.
+		forceudp = true;
+	}
+	else
+	{
+		// Cannot get here.
+		assert(1 && "Recursed from UDP fallback case");
+	}
+
+
 	if (cls.state != ca_connected)
 	{
 		return;
@@ -103,6 +156,14 @@ CL_RequestNextDownload(void)
 					precache_model_skin = 1;
 				}
 
+#ifdef USE_CURL
+				/* Wait for the models to download before checking * skins. */
+				if (CL_PendingHTTPDownloads())
+				{
+					return;
+				}
+#endif
+
 				/* checking for skins in the model */
 				if (!precache_model)
 				{
@@ -333,11 +394,30 @@ CL_RequestNextDownload(void)
 				}
 			}
 		}
-
-		/* precache phase completed */
-		precache_check = ENV_CNT;
 	}
 
+
+#ifdef USE_CURL
+	/* Wait for pending downloads. */
+	if (CL_PendingHTTPDownloads())
+	{
+		return;
+	}
+
+
+	if (CL_CheckHTTPError())
+	{
+		/* Mkay, there were download errors. Let's start over. */
+		precacherIteration++;
+		CL_ResetPrecacheCheck();
+		CL_RequestNextDownload();
+		return;
+	}
+#endif
+
+	/* precache phase completed */
+	precache_check = ENV_CNT;
+
 	if (precache_check == ENV_CNT)
 	{
 		precache_check = ENV_CNT + 1;
@@ -412,8 +492,21 @@ CL_RequestNextDownload(void)
 		precache_check = TEXTURE_CNT + 999;
 	}
 
-	CL_RegisterSounds();
+#ifdef USE_CURL
+	/* Wait for pending downloads. */
+	if (CL_PendingHTTPDownloads())
+	{
+		return;
+	}
+#endif
 
+	/* This map is done, start over for next map. */
+	forceudp = false;
+	precacherIteration = 0;
+	gamedirForFilelist = false;
+
+	CL_HTTP_EnableGenericFilelist();
+	CL_RegisterSounds();
 	CL_PrepRefresh();
 
 	MSG_WriteByte(&cls.netchan.message, clc_stringcmd);
@@ -464,6 +557,37 @@ CL_CheckOrDownloadFile(char *filename)
 		return true;
 	}
 
+#ifdef USE_CURL
+	if (!forceudp)
+	{
+		if (CL_QueueHTTPDownload(filename, gamedirForFilelist))
+		{
+			/* We return true so that the precache check
+			   keeps feeding us more files. Since we have
+			   multiple HTTP connections we want to
+			   minimize latency and be constantly sending
+			   requests, not one at a time. */
+			return true;
+		}
+	}
+	else
+	{
+		/* There're 2 cases:
+			- forceudp was set after a 404. In this case we
+			  want to retry that single file over UDP and
+			  all later files over HTTP.
+			- forceudp was set after another error code.
+			  In that case the HTTP code aborts all HTTP
+			  downloads and CL_QueueHTTPDownload() returns
+			  false. */
+		forceudp = false;
+
+		/* We might be connected to an r1q2-style HTTP server
+		   that missed just one file. So reset the precacher
+		   iteration counter to start over. */
+		precacherIteration = 0;
+	}
+#endif
 	strcpy(cls.downloadname, filename);
 
 	/* download to a temp name, and only rename
@@ -620,7 +744,7 @@ CL_ParseDownload(void)
 		/* rename the temp file to it's final name */
 		CL_DownloadFileName(oldn, sizeof(oldn), cls.downloadtempname);
 		CL_DownloadFileName(newn, sizeof(newn), cls.downloadname);
-		r = rename(oldn, newn);
+		r = Sys_Rename(oldn, newn);
 
 		if (r)
 		{
