@@ -26,7 +26,7 @@
  */
 
 #include "header/client.h"
-#include "../backends/generic/header/input.h"
+#include "input/header/input.h"
 
 void CL_ForwardToServer_f(void);
 void CL_Changing_f(void);
@@ -425,18 +425,30 @@ CL_Userinfo_f(void)
 void
 CL_Snd_Restart_f(void)
 {
+	OGG_SaveState();
+
 	S_Shutdown();
 	S_Init();
+
 	CL_RegisterSounds();
+
+	OGG_InitTrackList();
+	OGG_RecoverState();
 }
 
 int precache_check;
 int precache_spawncount;
 int precache_tex;
 int precache_model_skin;
-
 byte *precache_model;
 
+void CL_ResetPrecacheCheck (void)
+{
+	precache_check = CS_MODELS;
+	precache_model = 0;
+	precache_model_skin = 0;
+}
+
 /*
  * The server will send this command right
  * before allowing the client into the server
@@ -541,6 +553,13 @@ CL_InitLocal(void)
 
 	cl_vwep = Cvar_Get("cl_vwep", "1", CVAR_ARCHIVE);
 
+#ifdef USE_CURL
+	cl_http_proxy = Cvar_Get("cl_http_proxy", "", 0);
+	cl_http_filelists = Cvar_Get("cl_http_filelists", "1", 0);
+	cl_http_downloads = Cvar_Get("cl_http_downloads", "1", CVAR_ARCHIVE);
+	cl_http_max_connections = Cvar_Get("cl_http_max_connections", "4", 0);
+#endif
+
 	/* register our commands */
 	Cmd_AddCommand("cmd", CL_ForwardToServer_f);
 	Cmd_AddCommand("pause", CL_Pause_f);
@@ -757,7 +776,15 @@ CL_Frame(int packetdelta, int renderdelta, int timedelta, qboolean packetframe,
 		}
 	}
 
-	// Update input stuff
+	// Run HTTP downloads more often while connecting.
+#ifdef USE_CURL
+	if (cls.state == ca_connected)
+	{
+		CL_RunHTTPDownloads();
+	}
+#endif
+
+	// Update input stuff.
 	if (packetframe || renderframe)
 	{
 		CL_ReadPackets();
@@ -786,6 +813,11 @@ CL_Frame(int packetdelta, int renderdelta, int timedelta, qboolean packetframe,
 	{
 		CL_SendCmd();
 		CL_CheckForResend();
+
+		// Run HTTP downloads during game.
+#ifdef USE_CURL
+		CL_RunHTTPDownloads();
+#endif
 	}
 
 	if (renderframe)
@@ -814,10 +846,6 @@ CL_Frame(int packetdelta, int renderdelta, int timedelta, qboolean packetframe,
 		/* update audio */
 		S_Update(cl.refdef.vieworg, cl.v_forward, cl.v_right, cl.v_up);
 
-#ifdef CDA
-		CDAudio_Update();
-#endif
-
 		/* advance local effects for next frame */
 		CL_RunDLights();
 		CL_RunLightStyles();
@@ -884,12 +912,12 @@ CL_Init(void)
 
 	M_Init();
 
-	cls.disable_screen = true; /* don't draw yet */
-
-#ifdef CDA
-	CDAudio_Init();
+#ifdef USE_CURL
+	CL_InitHTTPDownloads();
 #endif
 
+	cls.disable_screen = true; /* don't draw yet */
+
 	CL_InitLocal();
 
 	Cbuf_Execute();
@@ -910,16 +938,16 @@ CL_Shutdown(void)
 
 	isdown = true;
 
+#ifdef USE_CURL
+	CL_HTTP_Cleanup(true);
+#endif
+
 	CL_WriteConfiguration();
 
 	Key_WriteConsoleHistory();
 
-#ifdef CDA
-	CDAudio_Shutdown();
-#endif
-#ifdef OGG
 	OGG_Stop();
-#endif
+
 	S_Shutdown();
 	IN_Shutdown();
 	VID_Shutdown();
