@@ -25,7 +25,7 @@
  */
 
 #include "header/client.h"
-#include "../backends/generic/header/input.h"
+#include "input/header/input.h"
 
 void CL_DownloadFileName(char *dest, int destlen, char *fn);
 void CL_ParseDownload(void);
@@ -1078,20 +1078,11 @@ CL_ParseConfigString(void)
 	{
 		CL_SetLightstyle(i - CS_LIGHTS);
 	}
-
 	else if (i == CS_CDTRACK)
 	{
 		if (cl.refresh_prepped)
 		{
-			int track = (int)strtol(cl.configstrings[CS_CDTRACK], (char **)NULL, 10);
-#ifdef CDA
-			CDAudio_Play(track, true);
-#endif
-
-#ifdef OGG
-
-			OGG_PlayTrack(track);
-#endif
+			OGG_PlayTrack((int)strtol(cl.configstrings[CS_CDTRACK], (char **)NULL, 10));
 		}
 	}
 	else if ((i >= CS_MODELS) && (i < CS_MODELS + MAX_MODELS))
