@@ -567,12 +567,8 @@ SCR_BeginLoadingPlaque(void)
 {
 	S_StopAllSounds();
 	cl.sound_prepped = false; /* don't play ambients */
-#ifdef CDA
-	CDAudio_Stop();
-#endif
-#ifdef OGG
+
 	OGG_Stop();
-#endif
 
 	if (cls.disable_screen)
 	{
