@@ -25,7 +25,7 @@
  */
 
 #include "header/client.h"
-#include "../backends/generic/header/input.h"
+#include "input/header/input.h"
 
 /* development tools for weapons */
 int gun_frame;
@@ -361,35 +361,17 @@ CL_PrepRefresh(void)
 	cl.refresh_prepped = true;
 	cl.force_refdef = true; /* make sure we have a valid refdef */
 
-#if defined(OGG) || defined(CDA)
-
+	/* start the cd track */
 	int track = (int)strtol(cl.configstrings[CS_CDTRACK], (char **)NULL, 10);
 
-	/* start the cd track */
-	if (Cvar_VariableValue("cd_shuffle"))
+	if (Cvar_VariableValue("ogg_shuffle"))
 	{
-#ifdef CDA
-		CDAudio_RandomPlay();
-#endif
-
-#ifdef OGG
 		OGG_PlayTrack(track);
-#endif
 	}
 	else
 	{
- #ifdef CDA
-		CDAudio_Play(track, (char **)NULL, 10), true);
- #endif
-
- #ifdef OGG
-
 		OGG_PlayTrack(track);
-
- #endif
 	}
-
-#endif
 }
 
 float
