@@ -47,16 +47,18 @@
 #include <stdlib.h>
 #include <ctype.h>
 
-#include "ref.h"
-#include "vid.h"
+#include "../../common/header/common.h"
+
+#include "../curl/header/download.h"
+#include "../sound/header/sound.h"
+#include "../sound/header/vorbis.h"
+#include "../vid/header/ref.h"
+#include "../vid/header/vid.h"
+
 #include "screen.h"
 #include "keyboard.h"
 #include "console.h"
 
-#include "../sound/header/cdaudio.h"
-#include "../sound/header/sound.h"
-#include "../sound/header/vorbis.h"
-
 typedef struct
 {
 	qboolean		valid; /* cleared if delta parsing was invalid */
@@ -239,12 +241,22 @@ typedef struct
 	char		downloadname[MAX_OSPATH];
 	int			downloadnumber;
 	dltype_t	downloadtype;
+	size_t		downloadposition;
 	int			downloadpercent;
 
 	/* demo recording info must be here, so it isn't cleared on level change */
 	qboolean	demorecording;
 	qboolean	demowaiting; /* don't record until a non-delta message is received */
 	FILE		*demofile;
+
+#ifdef USE_CURL
+	/* http downloading */
+	dlqueue_t  downloadQueue; /* queues with files to download. */
+	dlhandle_t HTTPHandles[MAX_HTTP_HANDLES]; /* download handles. */
+	char	   downloadServer[512]; /* URL prefix to dowload from .*/
+	char	   downloadServerRetry[512]; /* retry count. */
+	char	   downloadReferer[32]; /* referer string. */
+#endif
 } client_static_t;
 
 extern client_static_t	cls;
@@ -430,6 +442,7 @@ void CL_GetChallengePacket (void);
 void CL_PingServers_f (void);
 void CL_Snd_Restart_f (void);
 void CL_RequestNextDownload (void);
+void CL_ResetPrecacheCheck (void);
 
 typedef struct
 {
