@@ -188,103 +188,6 @@ enum QKEYS {
 	K_AUX31,
 	K_AUX32,
 
-	K_WORLD_0,
-	K_WORLD_1,
-	K_WORLD_2,
-	K_WORLD_3,
-	K_WORLD_4,
-	K_WORLD_5,
-	K_WORLD_6,
-	K_WORLD_7,
-	K_WORLD_8,
-	K_WORLD_9,
-	K_WORLD_10,
-	K_WORLD_11,
-	K_WORLD_12,
-	K_WORLD_13,
-	K_WORLD_14,
-	K_WORLD_15,
-	K_WORLD_16,
-	K_WORLD_17,
-	K_WORLD_18,
-	K_WORLD_19,
-	K_WORLD_20,
-	K_WORLD_21,
-	K_WORLD_22,
-	K_WORLD_23,
-	K_WORLD_24,
-	K_WORLD_25,
-	K_WORLD_26,
-	K_WORLD_27,
-	K_WORLD_28,
-	K_WORLD_29,
-	K_WORLD_30,
-	K_WORLD_31,
-	K_WORLD_32,
-	K_WORLD_33,
-	K_WORLD_34,
-	K_WORLD_35,
-	K_WORLD_36,
-	K_WORLD_37,
-	K_WORLD_38,
-	K_WORLD_39,
-	K_WORLD_40,
-	K_WORLD_41,
-	K_WORLD_42,
-	K_WORLD_43,
-	K_WORLD_44,
-	K_WORLD_45,
-	K_WORLD_46,
-	K_WORLD_47,
-	K_WORLD_48,
-	K_WORLD_49,
-	K_WORLD_50,
-	K_WORLD_51,
-	K_WORLD_52,
-	K_WORLD_53,
-	K_WORLD_54,
-	K_WORLD_55,
-	K_WORLD_56,
-	K_WORLD_57,
-	K_WORLD_58,
-	K_WORLD_59,
-	K_WORLD_60,
-	K_WORLD_61,
-	K_WORLD_62,
-	K_WORLD_63,
-	K_WORLD_64,
-	K_WORLD_65,
-	K_WORLD_66,
-	K_WORLD_67,
-	K_WORLD_68,
-	K_WORLD_69,
-	K_WORLD_70,
-	K_WORLD_71,
-	K_WORLD_72,
-	K_WORLD_73,
-	K_WORLD_74,
-	K_WORLD_75,
-	K_WORLD_76,
-	K_WORLD_77,
-	K_WORLD_78,
-	K_WORLD_79,
-	K_WORLD_80,
-	K_WORLD_81,
-	K_WORLD_82,
-	K_WORLD_83,
-	K_WORLD_84,
-	K_WORLD_85,
-	K_WORLD_86,
-	K_WORLD_87,
-	K_WORLD_88,
-	K_WORLD_89,
-	K_WORLD_90,
-	K_WORLD_91,
-	K_WORLD_92,
-	K_WORLD_93,
-	K_WORLD_94,
-	K_WORLD_95,
-
 	K_SUPER,
 	K_COMPOSE,
 	K_MODE,
@@ -292,9 +195,7 @@ enum QKEYS {
 	K_PRINT,
 	K_SYSREQ,
 	K_SCROLLOCK,
-	K_BREAK,
 	K_MENU,
-	K_EURO,
 	K_UNDO,
 
 	K_LAST
@@ -316,7 +217,7 @@ void Key_ReadConsoleHistory();
 void Key_WriteConsoleHistory();
 void Key_SetBinding(int keynum, char *binding);
 void Key_MarkAllUp(void);
-void Haptic_Feedback(char *name);
+void Haptic_Feedback(char *name, int effect_volume, int effect_x, int effect_y, int effect_z);
 int Key_GetMenuKey(int key);
 
 #endif
