@@ -66,7 +66,6 @@ float r_base_world_matrix[16];
 refdef_t r_newrefdef;
 
 int r_viewcluster, r_viewcluster2, r_oldviewcluster, r_oldviewcluster2;
-extern qboolean have_stencil;
 unsigned r_rawpalette[256];
 
 cvar_t *r_norefresh;
@@ -898,7 +897,7 @@ R_Clear(void)
 	}
 
 	/* stencilbuffer shadows */
-	if (gl_shadows->value && have_stencil && gl1_stencilshadow->value)
+	if (gl_shadows->value && gl_state.stencil && gl1_stencilshadow->value)
 	{
 		glClearStencil(1);
 		glClear(GL_STENCIL_BUFFER_BIT);
@@ -1394,17 +1393,6 @@ RI_Init()
 	R_Printf(PRINT_ALL, "Refresh: " REF_VERSION "\n");
 	R_Printf(PRINT_ALL, "Client: " YQ2VERSION "\n\n");
 
-	/* Options */
-	R_Printf(PRINT_ALL, "Refresher build options:\n");
-
-	R_Printf(PRINT_ALL, " + Retexturing support\n");
-
-#ifdef X11GAMMA
-	R_Printf(PRINT_ALL, " + Gamma via X11\n\n");
-#else
-	R_Printf(PRINT_ALL, " - Gamma via X11\n\n");
-#endif
-
 	Draw_GetPalette();
 
 	R_Register();
@@ -1412,13 +1400,6 @@ RI_Init()
 	/* initialize our QGL dynamic bindings */
 	QGL_Init();
 
-	/* initialize OS-specific parts of OpenGL */
-	if (!ri.GLimp_Init())
-	{
-		QGL_Shutdown();
-		return false;
-	}
-
 	/* set our "safe" mode */
 	gl_state.prev_mode = 4;
 	gl_state.stereo_mode = gl1_stereo->value;
@@ -1472,8 +1453,8 @@ RI_Init()
 
 	if (strstr(gl_config.extensions_string, "GL_ARB_point_parameters"))
 	{
-			qglPointParameterfARB = (void (APIENTRY *)(GLenum, GLfloat))GLimp_GetProcAddress ( "glPointParameterfARB" );
-			qglPointParameterfvARB = (void (APIENTRY *)(GLenum, const GLfloat *))GLimp_GetProcAddress ( "glPointParameterfvARB" );
+			qglPointParameterfARB = (void (APIENTRY *)(GLenum, GLfloat))RI_GetProcAddress ( "glPointParameterfARB" );
+			qglPointParameterfvARB = (void (APIENTRY *)(GLenum, const GLfloat *))RI_GetProcAddress ( "glPointParameterfvARB" );
 	}
 
 	gl_config.pointparameters = false;
@@ -1504,7 +1485,7 @@ RI_Init()
 		strstr(gl_config.extensions_string, "GL_EXT_shared_texture_palette"))
 	{
 			qglColorTableEXT = (void (APIENTRY *)(GLenum, GLenum, GLsizei, GLenum, GLenum, const GLvoid * ))
-					GLimp_GetProcAddress ("glColorTableEXT");
+					RI_GetProcAddress ("glColorTableEXT");
 	}
 
 	gl_config.palettedtexture = false;
@@ -1587,15 +1568,12 @@ RI_Shutdown(void)
 	R_ShutdownImages();
 
 	/* shutdown OS specific OpenGL stuff like contexts, etc.  */
-	RI_ShutdownWindow(false);
+	RI_ShutdownContext();
 
 	/* shutdown our QGL subsystem */
 	QGL_Shutdown();
 }
 
-extern void UpdateHardwareGamma();
-extern void RI_SetSwapInterval(void);
-
 void
 RI_BeginFrame(float camera_separation)
 {
@@ -1625,11 +1603,7 @@ RI_BeginFrame(float camera_separation)
 	if (vid_gamma->modified)
 	{
 		vid_gamma->modified = false;
-
-		if (gl_state.hwgamma)
-		{
-			UpdateHardwareGamma();
-		}
+		RI_UpdateGamma();
 	}
 
 	// Clamp overbrightbits
@@ -1724,7 +1698,7 @@ RI_BeginFrame(float camera_separation)
 	if (r_vsync->modified)
 	{
 		r_vsync->modified = false;
-		RI_SetSwapInterval();
+		RI_SetVsync();
 	}
 
 	/* clear screen if desired */
@@ -1895,7 +1869,7 @@ GetRefAPI(refimport_t imp)
 	re.Shutdown = RI_Shutdown;
 	re.PrepareForWindow = RI_PrepareForWindow;
 	re.InitContext = RI_InitContext;
-	re.ShutdownWindow = RI_ShutdownWindow;
+	re.ShutdownContext = RI_ShutdownContext;
 	re.IsVSyncActive = RI_IsVSyncActive;
 	re.BeginRegistration = RI_BeginRegistration;
 	re.RegisterModel = RI_RegisterModel;
