@@ -48,7 +48,7 @@ GL3_Mod_PointInLeaf(vec3_t p, gl3model_t *model)
 
 	if (!model || !model->nodes)
 	{
-		ri.Sys_Error(ERR_DROP, "Mod_PointInLeaf: bad model");
+		ri.Sys_Error(ERR_DROP, "%s: bad model", __func__);
 	}
 
 	node = model->nodes;
@@ -76,51 +76,6 @@ GL3_Mod_PointInLeaf(vec3_t p, gl3model_t *model)
 	return NULL; /* never reached */
 }
 
-static byte *
-Mod_DecompressVis(byte *in, gl3model_t *model)
-{
-	static byte decompressed[MAX_MAP_LEAFS / 8];
-	int c;
-	byte *out;
-	int row;
-
-	row = (model->vis->numclusters + 7) >> 3;
-	out = decompressed;
-
-	if (!in)
-	{
-		/* no vis info, so make all visible */
-		while (row)
-		{
-			*out++ = 0xff;
-			row--;
-		}
-
-		return decompressed;
-	}
-
-	do
-	{
-		if (*in)
-		{
-			*out++ = *in++;
-			continue;
-		}
-
-		c = in[1];
-		in += 2;
-
-		while (c)
-		{
-			*out++ = 0;
-			c--;
-		}
-	}
-	while (out - decompressed < row);
-
-	return decompressed;
-}
-
 byte*
 GL3_Mod_ClusterPVS(int cluster, gl3model_t *model)
 {
@@ -131,7 +86,7 @@ GL3_Mod_ClusterPVS(int cluster, gl3model_t *model)
 
 	return Mod_DecompressVis((byte *)model->vis +
 			model->vis->bitofs[cluster][DVIS_PVS],
-			model);
+			(model->vis->numclusters + 7) >> 3);
 }
 
 void
@@ -211,8 +166,8 @@ Mod_LoadVertexes(lump_t *l)
 
 	if (l->filelen % sizeof(*in))
 	{
-		ri.Sys_Error(ERR_DROP, "MOD_LoadBmodel: funny lump size in %s",
-				loadmodel->name);
+		ri.Sys_Error(ERR_DROP, "%s: funny lump size in %s",
+				__func__, loadmodel->name);
 	}
 
 	count = l->filelen / sizeof(*in);
@@ -229,20 +184,6 @@ Mod_LoadVertexes(lump_t *l)
 	}
 }
 
-static float
-Mod_RadiusFromBounds(vec3_t mins, vec3_t maxs)
-{
-	int i;
-	vec3_t corner;
-
-	for (i = 0; i < 3; i++)
-	{
-		corner[i] = fabs(mins[i]) > fabs(maxs[i]) ? fabs(mins[i]) : fabs(maxs[i]);
-	}
-
-	return VectorLength(corner);
-}
-
 static void
 Mod_LoadSubmodels(lump_t *l)
 {
@@ -254,8 +195,8 @@ Mod_LoadSubmodels(lump_t *l)
 
 	if (l->filelen % sizeof(*in))
 	{
-		ri.Sys_Error(ERR_DROP, "MOD_LoadBmodel: funny lump size in %s",
-				loadmodel->name);
+		ri.Sys_Error(ERR_DROP, "%s: funny lump size in %s",
+				__func__, loadmodel->name);
 	}
 
 	count = l->filelen / sizeof(*in);
@@ -292,8 +233,8 @@ Mod_LoadEdges(lump_t *l)
 
 	if (l->filelen % sizeof(*in))
 	{
-		ri.Sys_Error(ERR_DROP, "MOD_LoadBmodel: funny lump size in %s",
-				loadmodel->name);
+		ri.Sys_Error(ERR_DROP, "%s: funny lump size in %s",
+				__func__, loadmodel->name);
 	}
 
 	count = l->filelen / sizeof(*in);
@@ -322,8 +263,8 @@ Mod_LoadTexinfo(lump_t *l)
 
 	if (l->filelen % sizeof(*in))
 	{
-		ri.Sys_Error(ERR_DROP, "MOD_LoadBmodel: funny lump size in %s",
-				loadmodel->name);
+		ri.Sys_Error(ERR_DROP, "%s: funny lump size in %s",
+				__func__, loadmodel->name);
 	}
 
 	count = l->filelen / sizeof(*in);
@@ -451,8 +392,8 @@ Mod_LoadFaces(lump_t *l)
 
 	if (l->filelen % sizeof(*in))
 	{
-		ri.Sys_Error(ERR_DROP, "MOD_LoadBmodel: funny lump size in %s",
-				loadmodel->name);
+		ri.Sys_Error(ERR_DROP, "%s: funny lump size in %s",
+				__func__, loadmodel->name);
 	}
 
 	count = l->filelen / sizeof(*in);
@@ -486,7 +427,8 @@ Mod_LoadFaces(lump_t *l)
 
 		if ((ti < 0) || (ti >= loadmodel->numtexinfo))
 		{
-			ri.Sys_Error(ERR_DROP, "MOD_LoadBmodel: bad texinfo number");
+			ri.Sys_Error(ERR_DROP, "%s: bad texinfo number",
+					__func__);
 		}
 
 		out->texinfo = loadmodel->texinfo + ti;
@@ -564,8 +506,8 @@ Mod_LoadNodes(lump_t *l)
 
 	if (l->filelen % sizeof(*in))
 	{
-		ri.Sys_Error(ERR_DROP, "MOD_LoadBmodel: funny lump size in %s",
-				loadmodel->name);
+		ri.Sys_Error(ERR_DROP, "%s: funny lump size in %s",
+				__func__, loadmodel->name);
 	}
 
 	count = l->filelen / sizeof(*in);
@@ -618,8 +560,8 @@ Mod_LoadLeafs(lump_t *l)
 
 	if (l->filelen % sizeof(*in))
 	{
-		ri.Sys_Error(ERR_DROP, "MOD_LoadBmodel: funny lump size in %s",
-				loadmodel->name);
+		ri.Sys_Error(ERR_DROP, "%s: funny lump size in %s",
+				__func__, loadmodel->name);
 	}
 
 	count = l->filelen / sizeof(*in);
@@ -630,6 +572,8 @@ Mod_LoadLeafs(lump_t *l)
 
 	for (i = 0; i < count; i++, in++, out++)
 	{
+		unsigned firstleafface;
+
 		for (j = 0; j < 3; j++)
 		{
 			out->minmaxs[j] = LittleShort(in->mins[j]);
@@ -642,9 +586,16 @@ Mod_LoadLeafs(lump_t *l)
 		out->cluster = LittleShort(in->cluster);
 		out->area = LittleShort(in->area);
 
-		out->firstmarksurface = loadmodel->marksurfaces +
-								LittleShort(in->firstleafface);
-		out->nummarksurfaces = LittleShort(in->numleaffaces);
+		// make unsigned long from signed short
+		firstleafface = LittleShort(in->firstleafface) & 0xFFFF;
+		out->nummarksurfaces = LittleShort(in->numleaffaces) & 0xFFFF;
+
+		out->firstmarksurface = loadmodel->marksurfaces + firstleafface;
+		if ((firstleafface + out->nummarksurfaces) > loadmodel->nummarksurfaces)
+		{
+			ri.Sys_Error(ERR_DROP, "%s: wrong marksurfaces position in %s",
+				__func__, loadmodel->name);
+		}
 	}
 }
 
@@ -659,8 +610,8 @@ Mod_LoadMarksurfaces(lump_t *l)
 
 	if (l->filelen % sizeof(*in))
 	{
-		ri.Sys_Error(ERR_DROP, "MOD_LoadBmodel: funny lump size in %s",
-				loadmodel->name);
+		ri.Sys_Error(ERR_DROP, "%s: funny lump size in %s",
+				__func__, loadmodel->name);
 	}
 
 	count = l->filelen / sizeof(*in);
@@ -675,7 +626,7 @@ Mod_LoadMarksurfaces(lump_t *l)
 
 		if ((j < 0) || (j >= loadmodel->numsurfaces))
 		{
-			ri.Sys_Error(ERR_DROP, "Mod_ParseMarksurfaces: bad surface number");
+			ri.Sys_Error(ERR_DROP, "%s: bad surface number", __func__);
 		}
 
 		out[i] = loadmodel->surfaces + j;
@@ -692,16 +643,16 @@ Mod_LoadSurfedges(lump_t *l)
 
 	if (l->filelen % sizeof(*in))
 	{
-		ri.Sys_Error(ERR_DROP, "MOD_LoadBmodel: funny lump size in %s",
-				loadmodel->name);
+		ri.Sys_Error(ERR_DROP, "%s: funny lump size in %s",
+				__func__, loadmodel->name);
 	}
 
 	count = l->filelen / sizeof(*in);
 
 	if ((count < 1) || (count >= MAX_MAP_SURFEDGES))
 	{
-		ri.Sys_Error(ERR_DROP, "MOD_LoadBmodel: bad surfedges count in %s: %i",
-				loadmodel->name, count);
+		ri.Sys_Error(ERR_DROP, "%s: bad surfedges count in %s: %i",
+				__func__, loadmodel->name, count);
 	}
 
 	out = Hunk_Alloc(count * sizeof(*out));
@@ -728,8 +679,8 @@ Mod_LoadPlanes(lump_t *l)
 
 	if (l->filelen % sizeof(*in))
 	{
-		ri.Sys_Error(ERR_DROP, "MOD_LoadBmodel: funny lump size in %s",
-				loadmodel->name);
+		ri.Sys_Error(ERR_DROP, "%s: funny lump size in %s",
+				__func__, loadmodel->name);
 	}
 
 	count = l->filelen / sizeof(*in);
@@ -759,12 +710,21 @@ Mod_LoadPlanes(lump_t *l)
 }
 
 static void
-Mod_LoadBrushModel(gl3model_t *mod, void *buffer)
+Mod_LoadBrushModel(gl3model_t *mod, void *buffer, int modfilelen)
 {
 	int i;
 	dheader_t *header;
 	mmodel_t *bm;
 
+	/* Because Quake II is is sometimes so ... "optimized" this
+	 * is going to be somewhat dirty. The map data contains indices
+	 * that we're converting into pointers. Yeah. No comments. The
+	 * indices are 32 bit long, they just encode the offset between
+	 * the hunks base address and the position in the hunk, so 32 bit
+	 * pointers should be enough. But let's play save, waste some
+	 * allocations and just take the plattforms pointer size instead
+	 * of relying on assumptions. */
+	loadmodel->extradata = Hunk_Begin(modfilelen * sizeof(void*));
 	loadmodel->type = mod_brush;
 
 	if (loadmodel != mod_known)
@@ -778,8 +738,8 @@ Mod_LoadBrushModel(gl3model_t *mod, void *buffer)
 
 	if (i != BSPVERSION)
 	{
-		ri.Sys_Error(ERR_DROP, "Mod_LoadBrushModel: %s has wrong version number (%i should be %i)",
-				mod->name, i, BSPVERSION);
+		ri.Sys_Error(ERR_DROP, "%s: %s has wrong version number (%i should be %i)",
+				__func__, mod->name, i, BSPVERSION);
 	}
 
 	/* swap all the lumps */
@@ -821,7 +781,8 @@ Mod_LoadBrushModel(gl3model_t *mod, void *buffer)
 
 		if (starmod->firstnode >= loadmodel->numnodes)
 		{
-			ri.Sys_Error(ERR_DROP, "Inline model %i has bad firstnode", i);
+			ri.Sys_Error(ERR_DROP, "%s: Inline model %i has bad firstnode",
+					__func__, i);
 		}
 
 		VectorCopy(bm->maxs, starmod->maxs);
@@ -858,7 +819,7 @@ GL3_Mod_FreeAll(void)
 	}
 }
 
-extern void GL3_LoadMD2(gl3model_t *mod, void *buffer);
+extern void GL3_LoadMD2(gl3model_t *mod, void *buffer, int modfilelen);
 extern void GL3_LoadSP2(gl3model_t *mod, void *buffer, int modfilelen);
 
 /*
@@ -873,7 +834,7 @@ Mod_ForName(char *name, qboolean crash)
 
 	if (!name[0])
 	{
-		ri.Sys_Error(ERR_DROP, "Mod_ForName: NULL name");
+		ri.Sys_Error(ERR_DROP, "%s: NULL name", __func__);
 	}
 
 	/* inline models are grabbed only from worldmodel */
@@ -883,7 +844,8 @@ Mod_ForName(char *name, qboolean crash)
 
 		if ((i < 1) || !gl3_worldmodel || (i >= gl3_worldmodel->numsubmodels))
 		{
-			ri.Sys_Error(ERR_DROP, "bad inline model number");
+			ri.Sys_Error(ERR_DROP, "%s: bad inline model number",
+					__func__);
 		}
 
 		return &mod_inline[i];
@@ -931,7 +893,8 @@ Mod_ForName(char *name, qboolean crash)
 	{
 		if (crash)
 		{
-			ri.Sys_Error(ERR_DROP, "Mod_NumForName: %s not found", mod->name);
+			ri.Sys_Error(ERR_DROP, "%s: %s not found",
+					__func__, mod->name);
 		}
 
 		memset(mod->name, 0, sizeof(mod->name));
@@ -944,24 +907,20 @@ Mod_ForName(char *name, qboolean crash)
 	switch (LittleLong(*(unsigned *)buf))
 	{
 		case IDALIASHEADER:
-			loadmodel->extradata = Hunk_Begin(0x200000);
-			GL3_LoadMD2(mod, buf);
+			GL3_LoadMD2(mod, buf, modfilelen);
 			break;
 
 		case IDSPRITEHEADER:
-			loadmodel->extradata = Hunk_Begin(0x10000);
 			GL3_LoadSP2(mod, buf, modfilelen);
 			break;
 
 		case IDBSPHEADER:
-			loadmodel->extradata = Hunk_Begin(0x1000000);
-			Mod_LoadBrushModel(mod, buf);
+			Mod_LoadBrushModel(mod, buf, modfilelen);
 			break;
 
 		default:
-			ri.Sys_Error(ERR_DROP,
-				"Mod_NumForName: unknown fileid for %s",
-				mod->name);
+			ri.Sys_Error(ERR_DROP, "%s: unknown fileid for %s",
+					__func__, mod->name);
 			break;
 	}
 
