@@ -809,7 +809,7 @@ GL3_DrawWorld(void)
 {
 	entity_t ent;
 
-	if (!gl_drawworld->value)
+	if (!r_drawworld->value)
 	{
 		return;
 	}
@@ -855,7 +855,7 @@ GL3_MarkLeaves(void)
 
 	if ((gl3_oldviewcluster == gl3_viewcluster) &&
 		(gl3_oldviewcluster2 == gl3_viewcluster2) &&
-		!gl_novis->value &&
+		!r_novis->value &&
 		(gl3_viewcluster != -1))
 	{
 		return;
@@ -863,7 +863,7 @@ GL3_MarkLeaves(void)
 
 	/* development aid to let you run around
 	   and see exactly where the pvs ends */
-	if (gl_lockpvs->value)
+	if (r_lockpvs->value)
 	{
 		return;
 	}
@@ -872,7 +872,7 @@ GL3_MarkLeaves(void)
 	gl3_oldviewcluster = gl3_viewcluster;
 	gl3_oldviewcluster2 = gl3_viewcluster2;
 
-	if (gl_novis->value || (gl3_viewcluster == -1) || !gl3_worldmodel->vis)
+	if (r_novis->value || (gl3_viewcluster == -1) || !gl3_worldmodel->vis)
 	{
 		/* mark everything */
 		for (i = 0; i < gl3_worldmodel->numleafs; i++)
