@@ -17,7 +17,7 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 */
-// r_edge.c
+// sw_edge.c
 #include <limits.h>
 #include <stdint.h>
 #include "header/local.h"
@@ -29,12 +29,6 @@ low depth complexity -- 1 to 3 or so
 have a sentinal at both ends?
 */
 
-
-edge_t	*auxedges;
-edge_t	*r_edges, *edge_p, *edge_max;
-
-surf_t	*surfaces, *surface_p, *surf_max;
-
 // surfaces are generated in back to front order by the bsp, so if a surf
 // pointer is greater than another one, it should be drawn in front
 // surfaces[1] is the background, and is used as the active surface stack
@@ -61,15 +55,9 @@ static int	miplevel;
 
 float	scale_for_mip;
 
-// FIXME: should go away
-extern void	R_RotateBmodel (void);
-extern void	R_TransformFrustum (void);
-
 static void R_GenerateSpans (void);
 static void R_GenerateSpansBackward (void);
 
-static void R_LeadingEdge (edge_t *edge);
-static void R_LeadingEdgeBackwards (edge_t *edge);
 static void R_TrailingEdge (surf_t *surf, edge_t *edge);
 
 /*
@@ -85,13 +73,9 @@ EDGE SCANNING
 R_BeginEdgeFrame
 ==============
 */
-void R_BeginEdgeFrame (void)
+void
+R_BeginEdgeFrame (void)
 {
-	edge_p = r_edges;
-	edge_max = &r_edges[r_numallocatededges];
-
-	surface_p = &surfaces[2];	// background is surface 1,
-					//  surface 0 is a dummy
 	surfaces[1].spans = NULL;	// no background spans yet
 	surfaces[1].flags = SURF_DRAWBACKGROUND;
 
@@ -155,7 +139,8 @@ R_InsertNewEdges (edge_t *edgestoadd, edge_t *edgelist)
 R_RemoveEdges
 ==============
 */
-static void R_RemoveEdges (edge_t *pedge)
+static void
+R_RemoveEdges (edge_t *pedge)
 {
 
 	do
@@ -198,7 +183,7 @@ R_StepActiveU (edge_t *pedge)
 			// find out where the edge goes in the edge list
 			pwedge = pedge->prev->prev;
 
-			while (pwedge && (pwedge->u > pedge->u))
+			while (pwedge->u > pedge->u)
 			{
 				pwedge = pwedge->prev;
 			}
@@ -261,7 +246,7 @@ D_SurfSearchBackwards
 ==============
 */
 static surf_t*
-D_SurfSearchBackwards(surf_t *surf, surf_t *surf2)
+D_SurfSearchBackwards(const surf_t *surf, surf_t *surf2)
 {
 	do
 	{
@@ -286,7 +271,7 @@ R_LeadingEdgeBackwards
 ==============
 */
 static void
-R_LeadingEdgeBackwards (edge_t *edge)
+R_LeadingEdgeBackwards (const edge_t *edge)
 {
 	surf_t		*surf, *surf2;
 
@@ -298,8 +283,6 @@ R_LeadingEdgeBackwards (edge_t *edge)
 	// end edge)
 	if (++surf->spanstate == 1)
 	{
-		shift20_t	iu;
-
 		surf2 = surfaces[1].next;
 
 		// if it's two surfaces on the same plane, the one that's already
@@ -307,6 +290,8 @@ R_LeadingEdgeBackwards (edge_t *edge)
 		// must be two bmodels in the same leaf; don't care, because they'll
 		// never be farthest anyway
 		if (surf->key > surf2->key || (surf->insubmodel && (surf->key == surf2->key))) {
+			shift20_t	iu;
+
 			// emit a span (obscures current top)
 			iu = edge->u >> shift_size;
 
@@ -385,7 +370,7 @@ D_SurfSearchForward
 ==============
 */
 static surf_t*
-D_SurfSearchForward(surf_t *surf, surf_t *surf2)
+D_SurfSearchForward(const surf_t *surf, surf_t *surf2)
 {
 	do
 	{
@@ -406,12 +391,14 @@ R_LeadingEdgeSearch
 ==============
 */
 static surf_t*
-R_LeadingEdgeSearch (edge_t *edge, surf_t *surf, surf_t *surf2)
+R_LeadingEdgeSearch (const edge_t *edge, const surf_t *surf, surf_t *surf2)
 {
-	float	fu, newzi, testzi, newzitop, newzibottom;
+	float	testzi, newzitop;
 
 	do
 	{
+		float	fu, newzi, newzibottom;
+
 		surf2 = D_SurfSearchForward(surf, surf2);
 
 		if (surf->key != surf2->key)
@@ -426,8 +413,10 @@ R_LeadingEdgeSearch (edge_t *edge, surf_t *surf, surf_t *surf2)
 		testzi = surf2->d_ziorigin + fv*surf2->d_zistepv +
 				fu*surf2->d_zistepu;
 
-		if (newzibottom < testzi)
+		if (newzibottom >= testzi)
+		{
 			return surf2;
+		}
 
 		newzitop = newzi * 1.01;
 	}
@@ -436,18 +425,62 @@ R_LeadingEdgeSearch (edge_t *edge, surf_t *surf, surf_t *surf2)
 	return surf2;
 }
 
+/*
+==============
+R_InsertBeforeSurf
+==============
+*/
+static void
+R_InsertBeforeSurf(surf_t *surf, surf_t *surf2)
+{
+	surf->next = surf2;
+	surf->prev = surf2->prev;
+	surf2->prev->next = surf;
+	surf2->prev = surf;
+}
+
+/*
+==============
+R_EmitSpanBeforeTop
+==============
+*/
+static void
+R_EmitSpanBeforeTop(const edge_t *edge, surf_t *surf, surf_t *surf2)
+{
+	shift20_t	iu;
+
+	iu = edge->u >> shift_size;
+
+	if (iu > surf2->last_u)
+	{
+		espan_t	*span;
+
+		span = span_p++;
+		span->u = surf2->last_u;
+		span->count = iu - span->u;
+		span->v = current_iv;
+		span->pnext = surf2->spans;
+		surf2->spans = span;
+	}
+
+	// set last_u on the new span
+	surf->last_u = iu;
+
+	// insert before surf2
+	R_InsertBeforeSurf(surf, surf2);
+}
+
 /*
 ==============
 R_LeadingEdge
 ==============
 */
 static void
-R_LeadingEdge (edge_t *edge)
+R_LeadingEdge (const edge_t *edge)
 {
 	if (edge->surfs[1])
 	{
-		surf_t		*surf, *surf2;
-		shift20_t	iu;
+		surf_t		*surf;
 
 		// it's adding a new surface in, so find the correct place
 		surf = &surfaces[edge->surfs[1]];
@@ -457,10 +490,16 @@ R_LeadingEdge (edge_t *edge)
 		// end edge)
 		if (++surf->spanstate == 1)
 		{
+			surf_t		*surf2;
+
 			surf2 = surfaces[1].next;
 
 			if (surf->key < surf2->key)
-				goto newtop;
+			{
+				// emit a span (obscures current top)
+				R_EmitSpanBeforeTop(edge, surf, surf2);
+				return;
+			}
 
 			// if it's two surfaces on the same plane, the one that's already
 			// active is in front, so keep going unless it's a bmodel
@@ -479,7 +518,9 @@ R_LeadingEdge (edge_t *edge)
 
 				if (newzibottom >= testzi)
 				{
-					goto newtop;
+					// emit a span (obscures current top)
+					R_EmitSpanBeforeTop(edge, surf, surf2);
+					return;
 				}
 
 				newzitop = newzi * 1.01;
@@ -487,38 +528,16 @@ R_LeadingEdge (edge_t *edge)
 				{
 					if (surf->d_zistepu >= surf2->d_zistepu)
 					{
-						goto newtop;
+						// emit a span (obscures current top)
+						R_EmitSpanBeforeTop(edge, surf, surf2);
+						return;
 					}
 				}
 			}
 
 			surf2 = R_LeadingEdgeSearch (edge, surf, surf2);
-			goto gotposition;
-newtop:
-			// emit a span (obscures current top)
-			iu = edge->u >> shift_size;
-
-			if (iu > surf2->last_u)
-			{
-				espan_t	*span;
-
-				span = span_p++;
-				span->u = surf2->last_u;
-				span->count = iu - span->u;
-				span->v = current_iv;
-				span->pnext = surf2->spans;
-				surf2->spans = span;
-			}
-
-			// set last_u on the new span
-			surf->last_u = iu;
-
-gotposition:
 			// insert before surf2
-			surf->next = surf2;
-			surf->prev = surf2->prev;
-			surf2->prev->next = surf;
-			surf2->prev = surf;
+			R_InsertBeforeSurf(surf, surf2);
 		}
 	}
 }
@@ -585,7 +604,7 @@ R_GenerateSpansBackward (void)
 	R_CleanupSpan ();
 }
 
-static void D_DrawSurfaces (void);
+static void D_DrawSurfaces (surf_t *surface);
 
 /*
 ==============
@@ -600,7 +619,8 @@ Each surface has a linked list of its visible spans
 
 ==============
 */
-void R_ScanEdges (void)
+void
+R_ScanEdges (surf_t *surface)
 {
 	shift20_t	iv, bottom;
 	espan_t		*basespan_p;
@@ -667,13 +687,13 @@ void R_ScanEdges (void)
 
 		// flush the span list if we can't be sure we have enough spans left for
 		// the next scan
-		if (span_p > max_span_p)
+		if (span_p >= max_span_p)
 		{
 			// Draw stuff on screen
-			D_DrawSurfaces ();
+			D_DrawSurfaces (surface);
 
 			// clear the surface span pointers
-			for (s = &surfaces[1] ; s<surface_p ; s++)
+			for (s = &surfaces[1] ; s<surface ; s++)
 				s->spans = NULL;
 
 			span_p = basespan_p;
@@ -702,7 +722,7 @@ void R_ScanEdges (void)
 	(*pdrawfunc) ();
 
 	// draw whatever's left in the span list
-	D_DrawSurfaces ();
+	D_DrawSurfaces (surface);
 }
 
 
@@ -718,7 +738,6 @@ static msurface_t		*pface;
 static surfcache_t		*pcurrentcache;
 static vec3_t			transformed_modelorg;
 static vec3_t			world_transformed_modelorg;
-static vec3_t			local_modelorg;
 
 /*
 =============
@@ -754,20 +773,16 @@ Simple single color fill with no texture mapping
 ==============
 */
 static void
-D_FlatFillSurface (surf_t *surf, int color)
+D_FlatFillSurface (surf_t *surf, pixel_t color)
 {
 	espan_t	*span;
 
 	for (span=surf->spans ; span ; span=span->pnext)
 	{
 		pixel_t   *pdest;
-		shift20_t u, u2;
 
-		pdest = d_viewbuffer + r_screenwidth*span->v;
-		u = span->u;
-		u2 = span->u + span->count - 1;
-		for ( ; u <= u2 ; u++)
-			pdest[u] = color;
+		pdest = d_viewbuffer + vid.width*span->v + span->u;
+		memset(pdest,  color&0xFF, span->count * sizeof(pixel_t));
 	}
 }
 
@@ -857,7 +872,7 @@ D_TurbulentSurf
 =================
 */
 static void
-D_TurbulentSurf (surf_t *s)
+D_TurbulentSurf(surf_t *s)
 {
 	d_zistepu = s->d_zistepu;
 	d_zistepv = s->d_zistepv;
@@ -870,27 +885,29 @@ D_TurbulentSurf (surf_t *s)
 
 	if (s->insubmodel)
 	{
+		entity_t *currententity;
+		vec3_t local_modelorg;
+
 		// FIXME: we don't want to do all this for every polygon!
 		// TODO: store once at start of frame
-		currententity = s->entity;	//FIXME: make this passed in to
-									// R_RotateBmodel ()
+		currententity = s->entity;
 		VectorSubtract (r_origin, currententity->origin,
 				local_modelorg);
 		TransformVector (local_modelorg, transformed_modelorg);
 
-		R_RotateBmodel ();	// FIXME: don't mess with the frustum,
-							// make entity passed in
+		R_RotateBmodel(currententity);	// FIXME: don't mess with the frustum,
+						// make entity passed in
 	}
 
 	D_CalcGradients (pface);
 
 	//============
 	//PGM
-	// textures that aren't warping are just flowing. Use NonTurbulent8 instead
+	// textures that aren't warping are just flowing. Use NonTurbulentPow2 instead
 	if(!(pface->texinfo->flags & SURF_WARP))
-		NonTurbulent8 (s->spans);
+		NonTurbulentPow2 (s->spans);
 	else
-		Turbulent8 (s->spans);
+		TurbulentPow2 (s->spans);
 	//PGM
 	//============
 
@@ -903,7 +920,6 @@ D_TurbulentSurf (surf_t *s)
 		// FIXME: we don't want to do this every time!
 		// TODO: speed up
 		//
-		currententity = NULL;	// &r_worldentity;
 		VectorCopy (world_transformed_modelorg,
 					transformed_modelorg);
 		VectorCopy (base_vpn, vpn);
@@ -934,7 +950,7 @@ D_SkySurf (surf_t *s)
 
 	D_CalcGradients (pface);
 
-	D_DrawSpans16 (s->spans);
+	D_DrawSpansPow2 (s->spans);
 
 	// set up a gradient for the background surface that places it
 	// effectively at infinity distance from the viewpoint
@@ -955,20 +971,23 @@ Normal surface cached, texture mapped surface
 static void
 D_SolidSurf (surf_t *s)
 {
+	entity_t *currententity;
+
 	d_zistepu = s->d_zistepu;
 	d_zistepv = s->d_zistepv;
 	d_ziorigin = s->d_ziorigin;
 
 	if (s->insubmodel)
 	{
+		vec3_t local_modelorg;
+
 		// FIXME: we don't want to do all this for every polygon!
 		// TODO: store once at start of frame
-		currententity = s->entity;	// FIXME: make this passed in to
-						// R_RotateBmodel ()
-		VectorSubtract (r_origin, currententity->origin, local_modelorg);
-		TransformVector (local_modelorg, transformed_modelorg);
+		currententity = s->entity;
+		VectorSubtract(r_origin, currententity->origin, local_modelorg);
+		TransformVector(local_modelorg, transformed_modelorg);
 
-		R_RotateBmodel ();	// FIXME: don't mess with the frustum,
+		R_RotateBmodel(currententity);	// FIXME: don't mess with the frustum,
 					// make entity passed in
 	}
 	else
@@ -978,14 +997,14 @@ D_SolidSurf (surf_t *s)
 	miplevel = D_MipLevelForScale(s->nearzi * scale_for_mip * pface->texinfo->mipadjust);
 
 	// FIXME: make this passed in to D_CacheSurface
-	pcurrentcache = D_CacheSurface (pface, miplevel);
+	pcurrentcache = D_CacheSurface (currententity, pface, miplevel);
 
 	cacheblock = (pixel_t *)pcurrentcache->data;
 	cachewidth = pcurrentcache->width;
 
 	D_CalcGradients (pface);
 
-	D_DrawSpans16 (s->spans);
+	D_DrawSpansPow2 (s->spans);
 
 	D_DrawZSpans (s->spans);
 
@@ -1002,7 +1021,6 @@ D_SolidSurf (surf_t *s)
 		VectorCopy (base_vup, vup);
 		VectorCopy (base_vright, vright);
 		R_TransformFrustum ();
-		currententity = NULL;	//&r_worldentity;
 	}
 }
 
@@ -1014,12 +1032,12 @@ To allow developers to see the polygon carving of the world
 =============
 */
 static void
-D_DrawflatSurfaces (void)
+D_DrawflatSurfaces (surf_t *surface)
 {
 	surf_t			*s;
 	int color = 0;
 
-	for (s = &surfaces[1] ; s<surface_p ; s++)
+	for (s = &surfaces[1] ; s<surface ; s++)
 	{
 		if (!s->spans)
 			continue;
@@ -1046,10 +1064,8 @@ May be called more than once a frame if the surf list overflows (higher res)
 ==============
 */
 static void
-D_DrawSurfaces (void)
+D_DrawSurfaces (surf_t *surface)
 {
-	// currententity = NULL;
-	// &r_worldentity;
 	VectorSubtract (r_origin, vec3_origin, modelorg);
 	TransformVector (modelorg, transformed_modelorg);
 	VectorCopy (transformed_modelorg, world_transformed_modelorg);
@@ -1058,7 +1074,7 @@ D_DrawSurfaces (void)
 	{
 		surf_t *s;
 
-		for (s = &surfaces[1] ; s<surface_p ; s++)
+		for (s = &surfaces[1] ; s<surface ; s++)
 		{
 			if (!s->spans)
 				continue;
@@ -1076,9 +1092,8 @@ D_DrawSurfaces (void)
 		}
 	}
 	else
-		D_DrawflatSurfaces ();
+		D_DrawflatSurfaces (surface);
 
-	currententity = NULL;	//&r_worldentity;
 	VectorSubtract (r_origin, vec3_origin, modelorg);
 	R_TransformFrustum ();
 }
