@@ -26,13 +26,13 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 model_t	*loadmodel;
 
-static void Mod_LoadSpriteModel (model_t *mod, void *buffer);
-static void Mod_LoadBrushModel (model_t *mod, void *buffer);
-static void Mod_LoadAliasModel (model_t *mod, void *buffer);
+static void Mod_LoadSpriteModel(model_t *mod, void *buffer, int modfilelen);
+static void Mod_LoadBrushModel(model_t *mod, void *buffer, int modfilelen);
+static void Mod_LoadAliasModel(model_t *mod, void *buffer, int modfilelen);
 
 static byte	mod_novis[MAX_MAP_LEAFS/8];
 
-#define	MAX_MOD_KNOWN	256
+#define	MAX_MOD_KNOWN	512
 static model_t	mod_known[MAX_MOD_KNOWN];
 static int	mod_numknown;
 
@@ -40,7 +40,6 @@ static int	mod_numknown;
 static model_t	mod_inline[MAX_MOD_KNOWN];
 
 int	registration_sequence;
-static int	modfilelen;
 
 //===============================================================================
 
@@ -50,7 +49,8 @@ static int	modfilelen;
 Mod_Modellist_f
 ================
 */
-void Mod_Modellist_f (void)
+void
+Mod_Modellist_f (void)
 {
 	int		i;
 	model_t	*mod;
@@ -60,9 +60,17 @@ void Mod_Modellist_f (void)
 	R_Printf(PRINT_ALL,"Loaded models:\n");
 	for (i=0, mod=mod_known ; i < mod_numknown ; i++, mod++)
 	{
+		char *in_use = "";
+
+		if (mod->registration_sequence == registration_sequence)
+		{
+			in_use = "*";
+		}
+
 		if (!mod->name[0])
 			continue;
-		R_Printf(PRINT_ALL, "%8i : %s\n",mod->extradatasize, mod->name);
+		R_Printf(PRINT_ALL, "%8i : %s %s\n",
+			 mod->extradatasize, mod->name, in_use);
 		total += mod->extradatasize;
 	}
 	R_Printf(PRINT_ALL, "Total resident: %i\n", total);
@@ -73,7 +81,8 @@ void Mod_Modellist_f (void)
 Mod_Init
 ===============
 */
-void Mod_Init (void)
+void
+Mod_Init (void)
 {
 	memset (mod_novis, 0xff, sizeof(mod_novis));
 }
@@ -90,10 +99,12 @@ Mod_ForName (char *name, qboolean crash)
 {
 	model_t	*mod;
 	unsigned *buf;
-	int		i;
+	int	i, modfilelen;
 
 	if (!name[0])
-		ri.Sys_Error (ERR_DROP,"Mod_ForName: NULL name");
+	{
+		ri.Sys_Error(ERR_DROP, "%s: NULL name", __func__);
+	}
 
 	//
 	// inline models are grabbed only from worldmodel
@@ -102,7 +113,11 @@ Mod_ForName (char *name, qboolean crash)
 	{
 		i = atoi(name+1);
 		if (i < 1 || !r_worldmodel || i >= r_worldmodel->numsubmodels)
-			ri.Sys_Error (ERR_DROP, "bad inline model number");
+		{
+			ri.Sys_Error(ERR_DROP, "%s: bad inline model number",
+					__func__);
+		}
+
 		return &mod_inline[i];
 	}
 
@@ -124,7 +139,7 @@ Mod_ForName (char *name, qboolean crash)
 	if (i == mod_numknown)
 	{
 		if (mod_numknown == MAX_MOD_KNOWN)
-			ri.Sys_Error (ERR_DROP, "mod_numknown == MAX_MOD_KNOWN");
+			ri.Sys_Error(ERR_DROP, "mod_numknown == MAX_MOD_KNOWN");
 		mod_numknown++;
 	}
 	strcpy (mod->name, name);
@@ -136,7 +151,11 @@ Mod_ForName (char *name, qboolean crash)
 	if (!buf)
 	{
 		if (crash)
-			ri.Sys_Error (ERR_DROP,"Mod_NumForName: %s not found", mod->name);
+		{
+			ri.Sys_Error(ERR_DROP, "%s: %s not found",
+					__func__, mod->name);
+		}
+
 		memset (mod->name, 0, sizeof(mod->name));
 		return NULL;
 	}
@@ -152,28 +171,26 @@ Mod_ForName (char *name, qboolean crash)
 	switch (LittleLong(*(unsigned *)buf))
 	{
 	case IDALIASHEADER:
-		loadmodel->extradata = Hunk_Begin (0x200000);
-		Mod_LoadAliasModel (mod, buf);
+		Mod_LoadAliasModel(mod, buf, modfilelen);
 		break;
 
 	case IDSPRITEHEADER:
-		loadmodel->extradata = Hunk_Begin (0x10000);
-		Mod_LoadSpriteModel (mod, buf);
+		Mod_LoadSpriteModel(mod, buf, modfilelen);
 		break;
 
 	case IDBSPHEADER:
-		loadmodel->extradata = Hunk_Begin (0x1000000);
-		Mod_LoadBrushModel (mod, buf);
+		Mod_LoadBrushModel(mod, buf, modfilelen);
 		break;
 
 	default:
-		ri.Sys_Error (ERR_DROP,"Mod_NumForName: unknown fileid for %s", mod->name);
+		ri.Sys_Error(ERR_DROP, "%s: unknown fileid for %s",
+				__func__, mod->name);
 		break;
 	}
 
-	loadmodel->extradatasize = Hunk_End ();
+	loadmodel->extradatasize = Hunk_End();
 
-	ri.FS_FreeFile (buf);
+	ri.FS_FreeFile(buf);
 
 	return mod;
 }
@@ -184,12 +201,15 @@ Mod_ForName (char *name, qboolean crash)
 Mod_PointInLeaf
 ===============
 */
-mleaf_t *Mod_PointInLeaf (vec3_t p, model_t *model)
+mleaf_t *
+Mod_PointInLeaf (vec3_t p, model_t *model)
 {
 	mnode_t		*node;
 
 	if (!model || !model->nodes)
-		ri.Sys_Error (ERR_DROP, "Mod_PointInLeaf: bad model");
+	{
+		ri.Sys_Error(ERR_DROP, "%s: bad model", __func__);
+	}
 
 	node = model->nodes;
 	while (node->contents == -1)
@@ -209,64 +229,19 @@ mleaf_t *Mod_PointInLeaf (vec3_t p, model_t *model)
 }
 
 
-/*
-===================
-Mod_DecompressVis
-===================
-*/
-static byte *
-Mod_DecompressVis (byte *in, model_t *model)
-{
-	static byte	decompressed[MAX_MAP_LEAFS/8];
-	int		c;
-	byte	*out;
-	int		row;
-
-	row = (model->vis->numclusters+7)>>3;
-	out = decompressed;
-
-	if (!in)
-	{
-		// no vis info, so make all visible
-		while (row)
-		{
-			*out++ = 0xff;
-			row--;
-		}
-		return decompressed;
-	}
-
-	do
-	{
-		if (*in)
-		{
-			*out++ = *in++;
-			continue;
-		}
-
-		c = in[1];
-		in += 2;
-		while (c)
-		{
-			*out++ = 0;
-			c--;
-		}
-	} while (out - decompressed < row);
-
-	return decompressed;
-}
-
 /*
 ==============
 Mod_ClusterPVS
 ==============
 */
-byte *Mod_ClusterPVS (int cluster, model_t *model)
+byte *
+Mod_ClusterPVS (int cluster, model_t *model)
 {
 	if (cluster == -1 || !model->vis)
 		return mod_novis;
-	return Mod_DecompressVis ( (byte *)model->vis + model->vis->bitofs[cluster][DVIS_PVS],
-		model);
+	return Mod_DecompressVis ( (byte *)model->vis +
+		model->vis->bitofs[cluster][DVIS_PVS],
+		(model->vis->numclusters+7)>>3);
 }
 
 /*
@@ -300,7 +275,7 @@ Mod_LoadLighting (lump_t *l)
 		return;
 	}
 	size = l->filelen/3;
-	loadmodel->lightdata = Hunk_Alloc (size);
+	loadmodel->lightdata = Hunk_Alloc(size);
 	in = (void *)(mod_base + l->fileofs);
 	for (i=0 ; i<size ; i++, in+=3)
 	{
@@ -318,7 +293,8 @@ static int r_leaftovis[MAX_MAP_LEAFS];
 static int r_vistoleaf[MAX_MAP_LEAFS];
 static int r_numvisleafs;
 
-void	R_NumberLeafs (mnode_t *node)
+void
+R_NumberLeafs (mnode_t *node)
 {
 	if (node->contents != -1)
 	{
@@ -355,7 +331,7 @@ Mod_LoadVisibility (lump_t *l)
 		loadmodel->vis = NULL;
 		return;
 	}
-	loadmodel->vis = Hunk_Alloc ( l->filelen);
+	loadmodel->vis = Hunk_Alloc(l->filelen);
 	memcpy (loadmodel->vis, mod_base + l->fileofs, l->filelen);
 
 	loadmodel->vis->numclusters = LittleLong (loadmodel->vis->numclusters);
@@ -380,10 +356,15 @@ Mod_LoadVertexes (lump_t *l)
 	int			i, count;
 
 	in = (void *)(mod_base + l->fileofs);
+
 	if (l->filelen % sizeof(*in))
-		ri.Sys_Error (ERR_DROP,"MOD_LoadBmodel: funny lump size in %s",loadmodel->name);
+	{
+		ri.Sys_Error(ERR_DROP, "%s: funny lump size in %s",
+				__func__, loadmodel->name);
+	}
+
 	count = l->filelen / sizeof(*in);
-	out = Hunk_Alloc ( (count+8)*sizeof(*out));		// extra for skybox
+	out = Hunk_Alloc((count+8)*sizeof(*out));		// extra for skybox
         /*
          * PATCH: eliasm
          *
@@ -417,10 +398,15 @@ Mod_LoadSubmodels (lump_t *l)
 	int			i, j, count;
 
 	in = (void *)(mod_base + l->fileofs);
+
 	if (l->filelen % sizeof(*in))
-		ri.Sys_Error (ERR_DROP,"MOD_LoadBmodel: funny lump size in %s",loadmodel->name);
+	{
+		ri.Sys_Error(ERR_DROP, "%s: funny lump size in %s",
+				__func__, loadmodel->name);
+	}
+
 	count = l->filelen / sizeof(*in);
-	out = Hunk_Alloc ( count*sizeof(*out));
+	out = Hunk_Alloc(count*sizeof(*out));
 
 	loadmodel->submodels = out;
 	loadmodel->numsubmodels = count;
@@ -453,9 +439,13 @@ Mod_LoadEdges (lump_t *l)
 
 	in = (void *)(mod_base + l->fileofs);
 	if (l->filelen % sizeof(*in))
-		ri.Sys_Error (ERR_DROP,"MOD_LoadBmodel: funny lump size in %s",loadmodel->name);
+	{
+		ri.Sys_Error(ERR_DROP, "%s: funny lump size in %s",
+				__func__, loadmodel->name);
+	}
+
 	count = l->filelen / sizeof(*in);
-	out = Hunk_Alloc ( (count + 13) * sizeof(*out));	// extra for skybox
+	out = Hunk_Alloc((count + 13) * sizeof(*out));	// extra for skybox
 
 	loadmodel->edges = out;
 	loadmodel->numedges = count;
@@ -481,10 +471,15 @@ Mod_LoadTexinfo (lump_t *l)
 	char	name[MAX_QPATH];
 
 	in = (void *)(mod_base + l->fileofs);
+
 	if (l->filelen % sizeof(*in))
-		ri.Sys_Error (ERR_DROP,"Mod_LoadTexinfo: funny lump size in %s",loadmodel->name);
+	{
+		ri.Sys_Error(ERR_DROP, "%s: funny lump size in %s",
+				__func__, loadmodel->name);
+	}
+
 	count = l->filelen / sizeof(*in);
-	out = Hunk_Alloc ( (count+6)*sizeof(*out));	// extra for skybox
+	out = Hunk_Alloc((count+6)*sizeof(*out));	// extra for skybox
 
 	loadmodel->texinfo = out;
 	loadmodel->numtexinfo = count;
@@ -600,7 +595,9 @@ CalcSurfaceExtents (msurface_t *s)
 		if (s->extents[i] < 16)
 			s->extents[i] = 16;	// take at least one cache block
 		if ( !(tex->flags & (SURF_WARP|SURF_SKY)) && s->extents[i] > 256)
-			ri.Sys_Error (ERR_DROP,"Bad surface extents");
+		{
+			ri.Sys_Error(ERR_DROP, "%s: Bad surface extents", __func__);
+		}
 	}
 }
 
@@ -618,10 +615,15 @@ Mod_LoadFaces (lump_t *l)
 	int			i, count, surfnum;
 
 	in = (void *)(mod_base + l->fileofs);
+
 	if (l->filelen % sizeof(*in))
-		ri.Sys_Error (ERR_DROP,"MOD_LoadBmodel: funny lump size in %s",loadmodel->name);
+	{
+		ri.Sys_Error(ERR_DROP, "%s: funny lump size in %s",
+				__func__, loadmodel->name);
+	}
+
 	count = l->filelen / sizeof(*in);
-	out = Hunk_Alloc ( (count+6)*sizeof(*out));	// extra for skybox
+	out = Hunk_Alloc((count+6)*sizeof(*out));	// extra for skybox
 
 	loadmodel->surfaces = out;
 	loadmodel->numsurfaces = count;
@@ -633,7 +635,10 @@ Mod_LoadFaces (lump_t *l)
 		out->firstedge = LittleLong(in->firstedge);
 		out->numedges = LittleShort(in->numedges);
 		if (out->numedges < 3)
-			ri.Sys_Error (ERR_DROP,"Surface with %d edges", out->numedges);
+		{
+			ri.Sys_Error(ERR_DROP, "%s: Surface with %d edges",
+					__func__, out->numedges);
+		}
 		out->flags = 0;
 
 		planenum = LittleShort(in->planenum);
@@ -725,10 +730,15 @@ Mod_LoadNodes (lump_t *l)
 	mnode_t 	*out;
 
 	in = (void *)(mod_base + l->fileofs);
+
 	if (l->filelen % sizeof(*in))
-		ri.Sys_Error (ERR_DROP,"MOD_LoadBmodel: funny lump size in %s",loadmodel->name);
+	{
+		ri.Sys_Error(ERR_DROP, "%s: funny lump size in %s",
+				__func__, loadmodel->name);
+	}
+
 	count = l->filelen / sizeof(*in);
-	out = Hunk_Alloc ( count*sizeof(*out));
+	out = Hunk_Alloc(count*sizeof(*out));
 
 	loadmodel->nodes = out;
 	loadmodel->numnodes = count;
@@ -776,16 +786,23 @@ Mod_LoadLeafs (lump_t *l)
 	int			i, j, count;
 
 	in = (void *)(mod_base + l->fileofs);
+
 	if (l->filelen % sizeof(*in))
-		ri.Sys_Error (ERR_DROP,"MOD_LoadBmodel: funny lump size in %s",loadmodel->name);
+	{
+		ri.Sys_Error(ERR_DROP, "%s: funny lump size in %s",
+				__func__, loadmodel->name);
+	}
+
 	count = l->filelen / sizeof(*in);
-	out = Hunk_Alloc ( count*sizeof(*out));
+	out = Hunk_Alloc(count*sizeof(*out));
 
 	loadmodel->leafs = out;
 	loadmodel->numleafs = count;
 
 	for ( i=0 ; i<count ; i++, in++, out++)
 	{
+		unsigned firstleafface;
+
 		for (j=0 ; j<3 ; j++)
 		{
 			out->minmaxs[j] = LittleShort (in->mins[j]);
@@ -796,9 +813,16 @@ Mod_LoadLeafs (lump_t *l)
 		out->cluster = LittleShort(in->cluster);
 		out->area = LittleShort(in->area);
 
-		out->firstmarksurface = loadmodel->marksurfaces +
-			LittleShort(in->firstleafface);
-		out->nummarksurfaces = LittleShort(in->numleaffaces);
+		// make unsigned long from signed short
+		firstleafface = LittleShort(in->firstleafface) & 0xFFFF;
+		out->nummarksurfaces = LittleShort(in->numleaffaces) & 0xFFFF;
+
+		out->firstmarksurface = loadmodel->marksurfaces + firstleafface;
+		if ((firstleafface + out->nummarksurfaces) > loadmodel->nummarksurfaces)
+		{
+			ri.Sys_Error(ERR_DROP, "%s: wrong marksurfaces position in %s",
+				__func__, loadmodel->name);
+		}
 	}
 }
 
@@ -816,10 +840,15 @@ Mod_LoadMarksurfaces (lump_t *l)
 	msurface_t **out;
 
 	in = (void *)(mod_base + l->fileofs);
+
 	if (l->filelen % sizeof(*in))
-		ri.Sys_Error (ERR_DROP,"MOD_LoadBmodel: funny lump size in %s",loadmodel->name);
+	{
+		ri.Sys_Error(ERR_DROP, "%s: funny lump size in %s",
+				__func__, loadmodel->name);
+	}
+
 	count = l->filelen / sizeof(*in);
-	out = Hunk_Alloc ( count*sizeof(*out));
+	out = Hunk_Alloc(count*sizeof(*out));
 
 	loadmodel->marksurfaces = out;
 	loadmodel->nummarksurfaces = count;
@@ -829,7 +858,9 @@ Mod_LoadMarksurfaces (lump_t *l)
 		int j;
 		j = LittleShort(in[i]);
 		if (j >= loadmodel->numsurfaces)
-			ri.Sys_Error (ERR_DROP,"Mod_ParseMarksurfaces: bad surface number");
+		{
+			ri.Sys_Error(ERR_DROP, "%s: bad surface number", __func__);
+		}
 		out[i] = loadmodel->surfaces + j;
 	}
 }
@@ -846,10 +877,15 @@ Mod_LoadSurfedges (lump_t *l)
 	int		*in, *out;
 
 	in = (void *)(mod_base + l->fileofs);
+
 	if (l->filelen % sizeof(*in))
-		ri.Sys_Error (ERR_DROP,"MOD_LoadBmodel: funny lump size in %s",loadmodel->name);
+	{
+		ri.Sys_Error(ERR_DROP, "%s: funny lump size in %s",
+				__func__, loadmodel->name);
+	}
+
 	count = l->filelen / sizeof(*in);
-	out = Hunk_Alloc ( (count+24)*sizeof(*out));	// extra for skybox
+	out = Hunk_Alloc((count+24)*sizeof(*out));	// extra for skybox
 
 	loadmodel->surfedges = out;
 	loadmodel->numsurfedges = count;
@@ -872,10 +908,15 @@ Mod_LoadPlanes (lump_t *l)
 	int			count;
 
 	in = (void *)(mod_base + l->fileofs);
+
 	if (l->filelen % sizeof(*in))
-		ri.Sys_Error (ERR_DROP,"MOD_LoadBmodel: funny lump size in %s",loadmodel->name);
+	{
+		ri.Sys_Error(ERR_DROP, "%s: funny lump size in %s",
+				__func__, loadmodel->name);
+	}
+
 	count = l->filelen / sizeof(*in);
-	out = Hunk_Alloc ( (count+6)*sizeof(*out));		// extra for skybox
+	out = Hunk_Alloc((count+6)*sizeof(*out));		// extra for skybox
 
 	loadmodel->planes = out;
 	loadmodel->numplanes = count;
@@ -905,21 +946,34 @@ Mod_LoadBrushModel
 =================
 */
 static void
-Mod_LoadBrushModel (model_t *mod, void *buffer)
+Mod_LoadBrushModel(model_t *mod, void *buffer, int modfilelen)
 {
-	int			i;
+	int		i;
 	dheader_t	*header;
 	dmodel_t 	*bm;
 
+	/* Because Quake II is is sometimes so ... "optimized" this
+	 * is going to be somewhat dirty. The map data contains indices
+	 * that we're converting into pointers. Yeah. No comments. The
+	 * indices are 32 bit long, they just encode the offset between
+	 * the hunks base address and the position in the hunk, so 32 bit
+	 * pointers should be enough. But let's play save, waste some
+	 * allocations and just take the plattforms pointer size instead
+	 * of relying on assumptions. */
+	loadmodel->extradata = Hunk_Begin(modfilelen * sizeof(void*));
+
 	loadmodel->type = mod_brush;
 	if (loadmodel != mod_known)
-		ri.Sys_Error (ERR_DROP, "Loaded a brush model after the world");
+		ri.Sys_Error(ERR_DROP, "Loaded a brush model after the world");
 
 	header = (dheader_t *)buffer;
 
 	i = LittleLong (header->version);
 	if (i != BSPVERSION)
-		ri.Sys_Error (ERR_DROP,"Mod_LoadBrushModel: %s has wrong version number (%i should be %i)", mod->name, i, BSPVERSION);
+	{
+		ri.Sys_Error(ERR_DROP, "%s: %s has wrong version number (%i should be %i)",
+				__func__, mod->name, i, BSPVERSION);
+	}
 
 	// swap all the lumps
 	mod_base = (byte *)header;
@@ -959,7 +1013,10 @@ Mod_LoadBrushModel (model_t *mod, void *buffer)
 		starmod->nummodelsurfaces = bm->numfaces;
 		starmod->firstnode = bm->headnode;
 		if (starmod->firstnode >= loadmodel->numnodes)
-			ri.Sys_Error (ERR_DROP, "Inline model %i has bad firstnode", i);
+		{
+			ri.Sys_Error(ERR_DROP, "%s: Inline model %i has bad firstnode",
+					__func__, i);
+		}
 
 		VectorCopy (bm->maxs, starmod->maxs);
 		VectorCopy (bm->mins, starmod->mins);
@@ -985,46 +1042,74 @@ Mod_LoadAliasModel
 =================
 */
 static void
-Mod_LoadAliasModel (model_t *mod, void *buffer)
+Mod_LoadAliasModel(model_t *mod, void *buffer, int modfilelen)
 {
-	int					i, j;
-	dmdl_t				*pinmodel, *pheader;
-	dstvert_t			*pinst, *poutst;
-	dtriangle_t			*pintri, *pouttri;
-	int					*pincmd, *poutcmd;
-	int					version;
+	int		i, j;
+	dmdl_t		*pinmodel, *pheader;
+	dstvert_t	*pinst, *poutst;
+	dtriangle_t	*pintri, *pouttri;
+	int		*pincmd, *poutcmd;
+	int		version;
+	int		ofs_end;
 
 	pinmodel = (dmdl_t *)buffer;
 
 	version = LittleLong (pinmodel->version);
 	if (version != ALIAS_VERSION)
-		ri.Sys_Error (ERR_DROP, "%s has wrong version number (%i should be %i)",
-				 mod->name, version, ALIAS_VERSION);
+	{
+		ri.Sys_Error(ERR_DROP, "%s: %s has wrong version number (%i should be %i)",
+				__func__, mod->name, version, ALIAS_VERSION);
+	}
 
-	pheader = Hunk_Alloc (LittleLong(pinmodel->ofs_end));
+	ofs_end = LittleLong(pinmodel->ofs_end);
+	if (ofs_end < 0 || ofs_end > modfilelen)
+	{
+		ri.Sys_Error(ERR_DROP, "%s: model %s file size(%d) too small, should be %d", mod->name,
+				__func__, modfilelen, ofs_end);
+	}
+
+	mod->extradata = Hunk_Begin(modfilelen);
+	pheader = Hunk_Alloc(ofs_end);
 
 	// byte swap the header fields and sanity check
 	for (i=0 ; i<sizeof(dmdl_t)/4 ; i++)
 		((int *)pheader)[i] = LittleLong (((int *)buffer)[i]);
 
 	if (pheader->skinheight > MAX_LBM_HEIGHT)
-		ri.Sys_Error (ERR_DROP, "model %s has a skin taller than %d", mod->name,
-				   MAX_LBM_HEIGHT);
+	{
+		ri.Sys_Error(ERR_DROP, "%s: model %s has a skin taller than %d", mod->name,
+				__func__, MAX_LBM_HEIGHT);
+	}
 
 	if (pheader->num_xyz <= 0)
-		ri.Sys_Error (ERR_DROP, "model %s has no vertices", mod->name);
+	{
+		ri.Sys_Error(ERR_DROP, "%s: model %s has no vertices",
+				__func__, mod->name);
+	}
 
 	if (pheader->num_xyz > MAX_VERTS)
-		ri.Sys_Error (ERR_DROP, "model %s has too many vertices", mod->name);
+	{
+		ri.Sys_Error(ERR_DROP, "%s: model %s has too many vertices",
+				__func__, mod->name);
+	}
 
 	if (pheader->num_st <= 0)
-		ri.Sys_Error (ERR_DROP, "model %s has no st vertices", mod->name);
+	{
+		ri.Sys_Error(ERR_DROP, "%s: model %s has no st vertices",
+				__func__, mod->name);
+	}
 
 	if (pheader->num_tris <= 0)
-		ri.Sys_Error (ERR_DROP, "model %s has no triangles", mod->name);
+	{
+		ri.Sys_Error(ERR_DROP, "%s: model %s has no triangles",
+				__func__, mod->name);
+	}
 
 	if (pheader->num_frames <= 0)
-		ri.Sys_Error (ERR_DROP, "model %s has no frames", mod->name);
+	{
+		ri.Sys_Error(ERR_DROP, "%s: model %s has no frames",
+				__func__, mod->name);
+	}
 
 	//
 	// load base s and t vertices (not used in gl version)
@@ -1111,25 +1196,30 @@ Mod_LoadSpriteModel
 =================
 */
 static void
-Mod_LoadSpriteModel (model_t *mod, void *buffer)
+Mod_LoadSpriteModel(model_t *mod, void *buffer, int modfilelen)
 {
 	dsprite_t	*sprin, *sprout;
 	int			i;
 
 	sprin = (dsprite_t *)buffer;
-	sprout = Hunk_Alloc (modfilelen);
+	mod->extradata = Hunk_Begin(modfilelen);
+	sprout = Hunk_Alloc(modfilelen);
 
 	sprout->ident = LittleLong (sprin->ident);
 	sprout->version = LittleLong (sprin->version);
 	sprout->numframes = LittleLong (sprin->numframes);
 
 	if (sprout->version != SPRITE_VERSION)
-		ri.Sys_Error (ERR_DROP, "%s has wrong version number (%i should be %i)",
-				 mod->name, sprout->version, SPRITE_VERSION);
+	{
+		ri.Sys_Error(ERR_DROP, "%s: %s has wrong version number (%i should be %i)",
+				__func__, mod->name, sprout->version, SPRITE_VERSION);
+	}
 
 	if (sprout->numframes > MAX_MD2SKINS)
-		ri.Sys_Error (ERR_DROP, "%s has too many frames (%i > %i)",
-				 mod->name, sprout->numframes, MAX_MD2SKINS);
+	{
+		ri.Sys_Error(ERR_DROP, "%s: %s has too many frames (%i > %i)",
+				__func__, mod->name, sprout->numframes, MAX_MD2SKINS);
+	}
 
 	// byte swap everything
 	for (i=0 ; i<sprout->numframes ; i++)
@@ -1154,7 +1244,8 @@ RE_BeginRegistration
 Specifies the model that will be used as the world
 =====================
 */
-void RE_BeginRegistration (char *model)
+void
+RE_BeginRegistration (char *model)
 {
 	char	fullname[MAX_QPATH];
 	cvar_t	*flushmap;
@@ -1180,7 +1271,8 @@ RE_RegisterModel
 
 =====================
 */
-struct model_s *RE_RegisterModel (char *name)
+struct model_s *
+RE_RegisterModel (char *name)
 {
 	model_t	*mod;
 
@@ -1226,7 +1318,8 @@ RE_EndRegistration
 
 =====================
 */
-void RE_EndRegistration (void)
+void
+RE_EndRegistration (void)
 {
 	int	i;
 	model_t	*mod;
@@ -1253,7 +1346,8 @@ void RE_EndRegistration (void)
 Mod_Free
 ================
 */
-void Mod_Free (model_t *mod)
+void
+Mod_Free (model_t *mod)
 {
 	Hunk_Free (mod->extradata);
 	memset (mod, 0, sizeof(*mod));
@@ -1264,7 +1358,8 @@ void Mod_Free (model_t *mod)
 Mod_FreeAll
 ================
 */
-void Mod_FreeAll (void)
+void
+Mod_FreeAll (void)
 {
 	int		i;
 
