@@ -42,8 +42,6 @@ polydesc_t	r_polydesc;
 
 msurface_t	*r_alpha_surfaces;
 
-extern int	*r_turb_turb;
-
 static int	clip_current;
 vec5_t		r_clip_verts[2][MAXWORKINGVERTS+2];
 static emitpoint_t	outverts[MAXWORKINGVERTS+3];
@@ -56,7 +54,7 @@ static void R_DrawPoly(int iswater);
 ** R_DrawSpanletOpaque
 */
 static void
-R_DrawSpanletOpaque( void )
+R_DrawSpanletOpaque(const int *r_turb_turb)
 {
 	do
 	{
@@ -88,11 +86,11 @@ R_DrawSpanletOpaque( void )
 ** R_DrawSpanletTurbulentStipple33
 */
 static void
-R_DrawSpanletTurbulentStipple33( void )
+R_DrawSpanletTurbulentStipple33(const int *r_turb_turb)
 {
 	pixel_t		*pdest = s_spanletvars.pdest;
 	zvalue_t	*pz    = s_spanletvars.pz;
-	int		izi   = s_spanletvars.izi;
+	zvalue_t	izi   = s_spanletvars.izi;
 
 	if ( s_spanletvars.v & 1 )
 	{
@@ -147,13 +145,13 @@ R_DrawSpanletTurbulentStipple33( void )
 ** R_DrawSpanletTurbulentStipple66
 */
 static void
-R_DrawSpanletTurbulentStipple66( void )
+R_DrawSpanletTurbulentStipple66(const int *r_turb_turb)
 {
 	unsigned	btemp;
 	int		sturb, tturb;
 	pixel_t		*pdest = s_spanletvars.pdest;
 	zvalue_t	*pz    = s_spanletvars.pz;
-	int		izi   = s_spanletvars.izi;
+	zvalue_t	izi   = s_spanletvars.izi;
 
 	if ( !( s_spanletvars.v & 1 ) )
 	{
@@ -235,7 +233,7 @@ R_DrawSpanletTurbulentStipple66( void )
 ** R_DrawSpanletTurbulentBlended
 */
 static void
-R_DrawSpanletTurbulentBlended66( void )
+R_DrawSpanletTurbulentBlended66(const int *r_turb_turb)
 {
 	do
 	{
@@ -260,7 +258,7 @@ R_DrawSpanletTurbulentBlended66( void )
 }
 
 static void
-R_DrawSpanletTurbulentBlended33( void )
+R_DrawSpanletTurbulentBlended33(const int *r_turb_turb)
 {
 	do
 	{
@@ -288,7 +286,7 @@ R_DrawSpanletTurbulentBlended33( void )
 ** R_DrawSpanlet33
 */
 static void
-R_DrawSpanlet33( void )
+R_DrawSpanlet33(const int *r_turb_turb)
 {
 	do
 	{
@@ -317,7 +315,7 @@ R_DrawSpanlet33( void )
 }
 
 static void
-R_DrawSpanletConstant33( void )
+R_DrawSpanletConstant33(const int *r_turb_turb)
 {
 	do
 	{
@@ -336,7 +334,7 @@ R_DrawSpanletConstant33( void )
 ** R_DrawSpanlet66
 */
 static void
-R_DrawSpanlet66( void )
+R_DrawSpanlet66(const int *r_turb_turb)
 {
 	do
 	{
@@ -368,11 +366,11 @@ R_DrawSpanlet66( void )
 ** R_DrawSpanlet33Stipple
 */
 static void
-R_DrawSpanlet33Stipple( void )
+R_DrawSpanlet33Stipple(const int *r_turb_turb)
 {
 	pixel_t		*pdest	= s_spanletvars.pdest;
 	zvalue_t	*pz	= s_spanletvars.pz;
-	int		izi	= s_spanletvars.izi;
+	zvalue_t	izi	= s_spanletvars.izi;
 
 	if ( r_polydesc.stipple_parity ^ ( s_spanletvars.v & 1 ) )
 	{
@@ -428,12 +426,12 @@ R_DrawSpanlet33Stipple( void )
 ** R_DrawSpanlet66Stipple
 */
 static void
-R_DrawSpanlet66Stipple( void )
+R_DrawSpanlet66Stipple(const int *r_turb_turb)
 {
 	unsigned	btemp;
 	pixel_t		*pdest = s_spanletvars.pdest;
 	zvalue_t	*pz = s_spanletvars.pz;
-	int		izi = s_spanletvars.izi;
+	zvalue_t	izi = s_spanletvars.izi;
 
 	s_spanletvars.pdest += s_spanletvars.spancount;
 	s_spanletvars.pz    += s_spanletvars.spancount;
@@ -595,17 +593,18 @@ R_ClipPolyFace (int nump, clipplane_t *pclipplane)
 static void
 R_PolygonDrawSpans(espan_t *pspan, int iswater )
 {
-	int	count;
 	int	snext, tnext;
 	float	sdivz, tdivz, zi, z, du, dv, spancountminus1;
 	float	sdivzspanletstepu, tdivzspanletstepu, zispanletstepu;
+	int	*r_turb_turb;
 
 	s_spanletvars.pbase = cacheblock;
 
 	//PGM
 	if ( iswater & SURF_WARP)
 		r_turb_turb = sintable + ((int)(r_newrefdef.time*SPEED)&(CYCLE-1));
-	else if (iswater & SURF_FLOWING)
+	else
+		// iswater & SURF_FLOWING
 		r_turb_turb = blanktable;
 	//PGM
 
@@ -621,8 +620,10 @@ R_PolygonDrawSpans(espan_t *pspan, int iswater )
 
 	do
 	{
-		s_spanletvars.pdest   = d_viewbuffer + (r_screenwidth * pspan->v) + pspan->u;
-		s_spanletvars.pz      = d_pzbuffer + (d_zwidth * pspan->v) + pspan->u;
+		int	count;
+
+		s_spanletvars.pdest   = d_viewbuffer + (vid.width * pspan->v) + pspan->u;
+		s_spanletvars.pz      = d_pzbuffer + (vid.width * pspan->v) + pspan->u;
 		s_spanletvars.u       = pspan->u;
 		s_spanletvars.v       = pspan->v;
 		count = pspan->count;
@@ -739,7 +740,7 @@ R_PolygonDrawSpans(espan_t *pspan, int iswater )
 					s_spanletvars.t = s_spanletvars.t & ((CYCLE<<16)-1);
 				}
 
-				r_polydesc.drawspanlet();
+				r_polydesc.drawspanlet(r_turb_turb);
 
 				s_spanletvars.s = snext;
 				s_spanletvars.t = tnext;
@@ -910,7 +911,8 @@ R_PolygonScanRightEdge(espan_t *s_polygon_spans)
 ** R_ClipAndDrawPoly
 */
 // PGM - isturbulent was qboolean. changed to int to allow passing more flags
-void R_ClipAndDrawPoly ( float alpha, int isturbulent, qboolean textured )
+void
+R_ClipAndDrawPoly ( float alpha, int isturbulent, qboolean textured )
 {
 	vec_t		*pv;
 	int		i, nump;
@@ -981,7 +983,9 @@ void R_ClipAndDrawPoly ( float alpha, int isturbulent, qboolean textured )
 		if (nump < 3)
 			return;
 		if (nump > MAXWORKINGVERTS)
-			ri.Sys_Error(ERR_DROP, "R_ClipAndDrawPoly: too many points: %d", nump );
+		{
+			ri.Sys_Error(ERR_DROP, "%s: too many points: %d", __func__, nump);
+		}
 	}
 
 	// transform vertices into viewspace and project
@@ -1024,9 +1028,9 @@ void R_ClipAndDrawPoly ( float alpha, int isturbulent, qboolean textured )
 ** R_BuildPolygonFromSurface
 */
 static void
-R_BuildPolygonFromSurface(msurface_t *fa)
+R_BuildPolygonFromSurface(const entity_t *currententity, const model_t *currentmodel, msurface_t *fa)
 {
-	int			i, lnumverts;
+	int		i, lnumverts;
 	medge_t		*pedges, *r_pedge;
 	float		*vec;
 	vec5_t     *pverts;
@@ -1082,7 +1086,7 @@ R_BuildPolygonFromSurface(msurface_t *fa)
 	{
 		surfcache_t *scache;
 
-		scache = D_CacheSurface( fa, 0 );
+		scache = D_CacheSurface(currententity, fa, 0);
 
 		r_polydesc.pixels       = scache->data;
 		r_polydesc.pixel_width  = scache->width;
@@ -1150,7 +1154,8 @@ R_PolygonCalculateGradients (void)
 ** This should NOT be called externally since it doesn't do clipping!
 */
 // PGM - iswater was qboolean. changed to support passing more flags
-static void R_DrawPoly(int iswater)
+static void
+R_DrawPoly(int iswater)
 {
 	int		i, nump;
 	float		ymin, ymax;
@@ -1206,11 +1211,11 @@ static void R_DrawPoly(int iswater)
 /*
 ** R_DrawAlphaSurfaces
 */
-void R_DrawAlphaSurfaces( void )
+void
+R_DrawAlphaSurfaces(const entity_t *currententity)
 {
 	msurface_t *s = r_alpha_surfaces;
-
-	currentmodel = r_worldmodel;
+	const model_t *currentmodel = r_worldmodel;
 
 	modelorg[0] = -r_origin[0];
 	modelorg[1] = -r_origin[1];
@@ -1218,7 +1223,7 @@ void R_DrawAlphaSurfaces( void )
 
 	while ( s )
 	{
-		R_BuildPolygonFromSurface( s );
+		R_BuildPolygonFromSurface(currententity, currentmodel, s);
 
 		//=======
 		//PGM
@@ -1244,7 +1249,8 @@ void R_DrawAlphaSurfaces( void )
 /*
 ** R_IMFlatShadedQuad
 */
-void R_IMFlatShadedQuad( vec3_t a, vec3_t b, vec3_t c, vec3_t d, int color, float alpha )
+void
+R_IMFlatShadedQuad( vec3_t a, vec3_t b, vec3_t c, vec3_t d, int color, float alpha )
 {
 	vec3_t s0, s1;
 
