@@ -23,12 +23,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #include "header/local.h"
 
-static pixel_t	*r_turb_pbase, *r_turb_pdest;
-static int	r_turb_s, r_turb_t, r_turb_sstep, r_turb_tstep;
-int		*r_turb_turb;
-static int	r_turb_spancount;
 
-static void D_DrawTurbulent8Span (void);
+#define SPANSTEP_SHIFT	4
 
 byte	**warp_rowptr;
 int	*warp_column;
@@ -41,7 +37,8 @@ this performs a slight compression of the screen at the same time as
 the sine warp, to keep the edges from wrapping
 =============
 */
-void D_WarpScreen (void)
+void
+D_WarpScreen (void)
 {
 	int	w, h;
 	int	u,v;
@@ -93,52 +90,126 @@ void D_WarpScreen (void)
 	}
 }
 
+
 /*
 =============
-D_DrawTurbulent8Span
+D_DrawSpanGetStep
+
+Return safe span step for u/z and v/z
 =============
 */
-static void D_DrawTurbulent8Span (void)
+static int
+D_DrawSpanGetStep(float d_zistepu, float d_zistepv)
 {
-	do
+	int	spanzshift = SPANSTEP_SHIFT;
+	int	spanzshift_value = (1 << spanzshift);
+	float	d_zistepu_shifted, d_zistepv_shifted;
+
+	d_zistepu_shifted = d_zistepu * SHIFT16XYZ_MULT;
+	d_zistepv_shifted = d_zistepv * SHIFT16XYZ_MULT;
+
+	// check that we can draw parallel surfaces to screen surface
+	// (both ends have same z value)
+	if ((int)(d_zistepu_shifted * spanzshift_value) == 0 &&
+	    (int)(d_zistepv_shifted * spanzshift_value) == 0)
 	{
-		int sturb, tturb;
-		sturb = ((r_turb_s + r_turb_turb[(r_turb_t>>16)&(CYCLE-1)])>>16)&63;
-		tturb = ((r_turb_t + r_turb_turb[(r_turb_s>>16)&(CYCLE-1)])>>16)&63;
-		*r_turb_pdest++ = *(r_turb_pbase + (tturb<<6) + sturb);
-		r_turb_s += r_turb_sstep;
-		r_turb_t += r_turb_tstep;
-	} while (--r_turb_spancount > 0);
+		// search next safe value
+		do
+		{
+			spanzshift ++;
+			spanzshift_value <<= 1;
+
+			if ((int)(d_zistepu_shifted * spanzshift_value) != 0 ||
+			    (int)(d_zistepv_shifted * spanzshift_value) != 0)
+			{
+				// step back to last safe value
+				return spanzshift - 1;
+			}
+
+		}
+		while (spanzshift_value < vid.width);
+	}
+	return spanzshift;
 }
 
 /*
 =============
-Turbulent8
+D_DrawZSpanGetStepValue
+
+Return safe span step for izistep
 =============
 */
-void Turbulent8 (espan_t *pspan)
+static int
+D_DrawZSpanGetStepValue(zvalue_t izistep)
+{
+	// check that we can draw parallel surfaces to screen surface
+	// (both ends have same z value)
+	int	count = 1;
+
+	while ((izistep * count) >> SHIFT16XYZ == 0 && count < vid.width)
+	{
+		count <<= 1;
+	}
+
+	return count;
+}
+
+/*
+=============
+D_DrawTurbulentSpan
+=============
+*/
+static pixel_t *
+D_DrawTurbulentSpan (pixel_t *pdest, const pixel_t *pbase,
+			 int s, int t,
+			 int sstep, int tstep,
+			 int spancount,
+			 int *turb)
+{
+	do
+	{
+		int sturb, tturb;
+		sturb = ((s + turb[(t>>SHIFT16XYZ)&(CYCLE-1)])>>16)&63;
+		tturb = ((t + turb[(s>>SHIFT16XYZ)&(CYCLE-1)])>>16)&63;
+		*pdest++ = *(pbase + (tturb<<6) + sturb);
+		s += sstep;
+		t += tstep;
+	} while (--spancount > 0);
+	return pdest;
+}
+
+/*
+=============
+TurbulentPow2
+=============
+*/
+void
+TurbulentPow2 (espan_t *pspan)
 {
-	int		snext, tnext;
 	float	spancountminus1;
-	float	sdivz16stepu, tdivz16stepu, zi16stepu;
+	float	sdivzpow2stepu, tdivzpow2stepu, zipow2stepu;
+	pixel_t	*r_turb_pbase;
+	int	*r_turb_turb;
+	int	spanstep_shift, spanstep_value;
+
+	spanstep_shift = D_DrawSpanGetStep(d_zistepu, d_zistepv);
+	spanstep_value = (1 << spanstep_shift);
 
 	r_turb_turb = sintable + ((int)(r_newrefdef.time*SPEED)&(CYCLE-1));
 
-	r_turb_sstep = 0;	// keep compiler happy
-	r_turb_tstep = 0;	// ditto
-
 	r_turb_pbase = (unsigned char *)cacheblock;
 
-	sdivz16stepu = d_sdivzstepu * 16;
-	tdivz16stepu = d_tdivzstepu * 16;
-	zi16stepu = d_zistepu * 16;
+	sdivzpow2stepu = d_sdivzstepu * spanstep_value;
+	tdivzpow2stepu = d_tdivzstepu * spanstep_value;
+	zipow2stepu = d_zistepu * spanstep_value;
 
 	do
 	{
-		int count;
+		int count, r_turb_s, r_turb_t;
 		float sdivz, tdivz, zi, z, du, dv;
+		pixel_t	*r_turb_pdest;
 
-		r_turb_pdest = d_viewbuffer + (r_screenwidth * pspan->v) + pspan->u;
+		r_turb_pdest = d_viewbuffer + (vid.width * pspan->v) + pspan->u;
 
 		count = pspan->count;
 
@@ -165,9 +236,16 @@ void Turbulent8 (espan_t *pspan)
 
 		do
 		{
+			int snext, tnext;
+			int r_turb_sstep, r_turb_tstep;
+			int r_turb_spancount;
+
+			r_turb_sstep = 0;	// keep compiler happy
+			r_turb_tstep = 0;	// ditto
+
 			// calculate s and t at the far end of the span
-			if (count >= 16)
-				r_turb_spancount = 16;
+			if (count >= spanstep_value)
+				r_turb_spancount = spanstep_value;
 			else
 				r_turb_spancount = count;
 
@@ -177,27 +255,29 @@ void Turbulent8 (espan_t *pspan)
 			{
 				// calculate s/z, t/z, zi->fixed s and t at far end of span,
 				// calculate s and t steps across span by shifting
-				sdivz += sdivz16stepu;
-				tdivz += tdivz16stepu;
-				zi += zi16stepu;
+				sdivz += sdivzpow2stepu;
+				tdivz += tdivzpow2stepu;
+				zi += zipow2stepu;
 				z = (float)SHIFT16XYZ_MULT / zi;	// prescale to 16.16 fixed-point
 
 				snext = (int)(sdivz * z) + sadjust;
 				if (snext > bbextents)
 					snext = bbextents;
-				else if (snext < 16)
-					snext = 16;	// prevent round-off error on <0 steps from
-							//  from causing overstepping & running off the
-							//  edge of the texture
+				else if (snext < spanstep_value)
+					// prevent round-off error on <0 steps from
+					//  from causing overstepping & running off the
+					//  edge of the texture
+					snext = spanstep_value;
 
 				tnext = (int)(tdivz * z) + tadjust;
 				if (tnext > bbextentt)
 					tnext = bbextentt;
-				else if (tnext < 16)
-					tnext = 16;	// guard against round-off error on <0 steps
+				else if (tnext < spanstep_value)
+					// guard against round-off error on <0 steps
+					tnext = spanstep_value;
 
-				r_turb_sstep = (snext - r_turb_s) >> 4;
-				r_turb_tstep = (tnext - r_turb_t) >> 4;
+				r_turb_sstep = (snext - r_turb_s) >> spanstep_shift;
+				r_turb_tstep = (tnext - r_turb_t) >> spanstep_shift;
 			}
 			else
 			{
@@ -213,16 +293,18 @@ void Turbulent8 (espan_t *pspan)
 				snext = (int)(sdivz * z) + sadjust;
 				if (snext > bbextents)
 					snext = bbextents;
-				else if (snext < 16)
-					snext = 16;	// prevent round-off error on <0 steps from
-							//  from causing overstepping & running off the
-							//  edge of the texture
+				else if (snext < spanstep_value)
+					// prevent round-off error on <0 steps from
+					//  from causing overstepping & running off the
+					//  edge of the texture
+					snext = spanstep_value;
 
 				tnext = (int)(tdivz * z) + tadjust;
 				if (tnext > bbextentt)
 					tnext = bbextentt;
-				else if (tnext < 16)
-					tnext = 16;	// guard against round-off error on <0 steps
+				else if (tnext < spanstep_value)
+					// guard against round-off error on <0 steps
+					tnext = spanstep_value;
 
 				if (r_turb_spancount > 1)
 				{
@@ -231,10 +313,14 @@ void Turbulent8 (espan_t *pspan)
 				}
 			}
 
-			r_turb_s = r_turb_s & ((CYCLE<<16)-1);
-			r_turb_t = r_turb_t & ((CYCLE<<16)-1);
+			r_turb_s = r_turb_s & ((CYCLE<<SHIFT16XYZ)-1);
+			r_turb_t = r_turb_t & ((CYCLE<<SHIFT16XYZ)-1);
 
-			D_DrawTurbulent8Span ();
+			r_turb_pdest = D_DrawTurbulentSpan (r_turb_pdest, r_turb_pbase,
+							    r_turb_s, r_turb_t,
+							    r_turb_sstep, r_turb_tstep,
+							    r_turb_spancount,
+							    r_turb_turb);
 
 			r_turb_s = snext;
 			r_turb_t = tnext;
@@ -248,33 +334,37 @@ void Turbulent8 (espan_t *pspan)
 //PGM
 /*
 =============
-NonTurbulent8 - this is for drawing scrolling textures. they're warping water textures
+NonTurbulentPow2 - this is for drawing scrolling textures. they're warping water textures
 	but the turbulence is automatically 0.
 =============
 */
-void NonTurbulent8 (espan_t *pspan)
+void
+NonTurbulentPow2 (espan_t *pspan)
 {
-	int snext, tnext;
 	float spancountminus1;
-	float sdivz16stepu, tdivz16stepu, zi16stepu;
+	float sdivzpow2stepu, tdivzpow2stepu, zipow2stepu;
+	pixel_t	*r_turb_pbase;
+	int	*r_turb_turb;
+	int	spanstep_shift, spanstep_value;
+
+	spanstep_shift = D_DrawSpanGetStep(d_zistepu, d_zistepv);
+	spanstep_value = (1 << spanstep_shift);
 
 	r_turb_turb = blanktable;
 
-	r_turb_sstep = 0;	// keep compiler happy
-	r_turb_tstep = 0;	// ditto
-
 	r_turb_pbase = (unsigned char *)cacheblock;
 
-	sdivz16stepu = d_sdivzstepu * 16;
-	tdivz16stepu = d_tdivzstepu * 16;
-	zi16stepu = d_zistepu * 16;
+	sdivzpow2stepu = d_sdivzstepu * spanstep_value;
+	tdivzpow2stepu = d_tdivzstepu * spanstep_value;
+	zipow2stepu = d_zistepu * spanstep_value;
 
 	do
 	{
-		int count;
+		int count, r_turb_s, r_turb_t;
 		float sdivz, tdivz, zi, z, dv, du;
+		pixel_t	*r_turb_pdest;
 
-		r_turb_pdest = d_viewbuffer + (r_screenwidth * pspan->v) + pspan->u;
+		r_turb_pdest = d_viewbuffer + (vid.width * pspan->v) + pspan->u;
 
 		count = pspan->count;
 
@@ -301,9 +391,16 @@ void NonTurbulent8 (espan_t *pspan)
 
 		do
 		{
+			int snext, tnext;
+			int r_turb_sstep, r_turb_tstep;
+			int r_turb_spancount;
+
+			r_turb_sstep = 0;	// keep compiler happy
+			r_turb_tstep = 0;	// ditto
+
 			// calculate s and t at the far end of the span
-			if (count >= 16)
-				r_turb_spancount = 16;
+			if (count >= spanstep_value)
+				r_turb_spancount = spanstep_value;
 			else
 				r_turb_spancount = count;
 
@@ -313,27 +410,29 @@ void NonTurbulent8 (espan_t *pspan)
 			{
 				// calculate s/z, t/z, zi->fixed s and t at far end of span,
 				// calculate s and t steps across span by shifting
-				sdivz += sdivz16stepu;
-				tdivz += tdivz16stepu;
-				zi += zi16stepu;
+				sdivz += sdivzpow2stepu;
+				tdivz += tdivzpow2stepu;
+				zi += zipow2stepu;
 				z = (float)SHIFT16XYZ_MULT / zi;	// prescale to 16.16 fixed-point
 
 				snext = (int)(sdivz * z) + sadjust;
 				if (snext > bbextents)
 					snext = bbextents;
-				else if (snext < 16)
-					snext = 16;	// prevent round-off error on <0 steps from
-							//  from causing overstepping & running off the
-							//  edge of the texture
+				else if (snext < spanstep_value)
+					// prevent round-off error on <0 steps from
+					//  from causing overstepping & running off the
+					//  edge of the texture
+					snext = spanstep_value;
 
 				tnext = (int)(tdivz * z) + tadjust;
 				if (tnext > bbextentt)
 					tnext = bbextentt;
-				else if (tnext < 16)
-					tnext = 16;	// guard against round-off error on <0 steps
+				else if (tnext < spanstep_value)
+					// guard against round-off error on <0 steps
+					tnext = spanstep_value;
 
-				r_turb_sstep = (snext - r_turb_s) >> 4;
-				r_turb_tstep = (tnext - r_turb_t) >> 4;
+				r_turb_sstep = (snext - r_turb_s) >> spanstep_shift;
+				r_turb_tstep = (tnext - r_turb_t) >> spanstep_shift;
 			}
 			else
 			{
@@ -349,16 +448,18 @@ void NonTurbulent8 (espan_t *pspan)
 				snext = (int)(sdivz * z) + sadjust;
 				if (snext > bbextents)
 					snext = bbextents;
-				else if (snext < 16)
-					snext = 16;	// prevent round-off error on <0 steps from
-							//  from causing overstepping & running off the
-							//  edge of the texture
+				else if (snext < spanstep_value)
+					// prevent round-off error on <0 steps from
+					//  from causing overstepping & running off the
+					//  edge of the texture
+					snext = spanstep_value;
 
 				tnext = (int)(tdivz * z) + tadjust;
 				if (tnext > bbextentt)
 					tnext = bbextentt;
-				else if (tnext < 16)
-					tnext = 16;	// guard against round-off error on <0 steps
+				else if (tnext < spanstep_value)
+					// guard against round-off error on <0 steps
+					tnext = spanstep_value;
 
 				if (r_turb_spancount > 1)
 				{
@@ -367,10 +468,14 @@ void NonTurbulent8 (espan_t *pspan)
 				}
 			}
 
-			r_turb_s = r_turb_s & ((CYCLE<<16)-1);
-			r_turb_t = r_turb_t & ((CYCLE<<16)-1);
+			r_turb_s = r_turb_s & ((CYCLE<<SHIFT16XYZ)-1);
+			r_turb_t = r_turb_t & ((CYCLE<<SHIFT16XYZ)-1);
 
-			D_DrawTurbulent8Span ();
+			r_turb_pdest = D_DrawTurbulentSpan (r_turb_pdest, r_turb_pbase,
+							    r_turb_s, r_turb_t,
+							    r_turb_sstep, r_turb_tstep,
+							    r_turb_spancount,
+							    r_turb_turb);
 
 			r_turb_s = snext;
 			r_turb_t = tnext;
@@ -384,48 +489,135 @@ void NonTurbulent8 (espan_t *pspan)
 
 // Enable custom filtering
 extern cvar_t	*sw_texture_filtering;
-static int filtering_kernel[2][2][2] = {
+static const int filtering_kernel[2][2][2] = {
 	{
-		{16384,	0},
-		{49152,	32768}
+		{0x1 << (SHIFT16XYZ-2), 0x0},
+		{0xC << (SHIFT16XYZ-4), 0x1 << (SHIFT16XYZ-1)}
 	},
 	{
-		{32768,	49152},
-		{0,	16384}
+		{0x1 << (SHIFT16XYZ-1), 0xC << (SHIFT16XYZ-4)},
+		{0x0, 0x1 << (SHIFT16XYZ-2)}
 	}
 };
 
 /*
 =============
-D_DrawSpans16
-
-  FIXME: actually make this subdivide by 16 instead of 8!!!
+D_DrawSpan
 =============
 */
-void D_DrawSpans16 (espan_t *pspan)
+static pixel_t *
+D_DrawSpan(pixel_t *pdest, const pixel_t *pbase, int s, int t, int sstep, int tstep, int spancount)
 {
-	int spancount;
-	unsigned char *pbase;
-	int snext, tnext, sstep, tstep;
-	float spancountminus1;
-	float sdivz8stepu, tdivz8stepu, zi8stepu;
+	const pixel_t *tdest_max = pdest + spancount;
 
-	sstep = 0;	// keep compiler happy
-	tstep = 0;	// ditto
+	// horisontal span (span in same row)
+	if (((t + tstep * spancount) >> SHIFT16XYZ) == (t >> SHIFT16XYZ))
+	{
+		// position in texture
+		const pixel_t *tbase = pbase + (t >> SHIFT16XYZ) * cachewidth;
+
+		do
+		{
+			*pdest++ = *(tbase + (s >> SHIFT16XYZ));
+			s += sstep;
+		} while (pdest < tdest_max);
+	}
+	// vertical span (span in same column)
+	else if (((s + sstep * spancount) >> SHIFT16XYZ) == (s >> SHIFT16XYZ))
+	{
+		// position in texture
+		const pixel_t *tbase = pbase + (s >> SHIFT16XYZ);
+
+		do
+		{
+			*pdest++ = *(tbase + (t >> SHIFT16XYZ) * cachewidth);
+			t += tstep;
+		} while (pdest < tdest_max);
+	}
+	// diagonal span
+	else
+	{
+		do
+		{
+			*pdest++ = *(pbase + (s >> SHIFT16XYZ) + (t >> SHIFT16XYZ) * cachewidth);
+			s += sstep;
+			t += tstep;
+		} while (pdest < tdest_max);
+	}
+
+	return pdest;
+}
+
+/*
+=============
+D_DrawSpanFiltered
+=============
+*/
+static pixel_t *
+D_DrawSpanFiltered(pixel_t *pdest, pixel_t *pbase, int s, int t, int sstep, int tstep, int spancount,
+	   const espan_t *pspan)
+{
+	do
+	{
+		int idiths = s;
+		int iditht = t;
+
+		int X = (pspan->u + spancount) & 1;
+		int Y = (pspan->v)&1;
+
+		//Using the kernel
+		idiths += filtering_kernel[X][Y][0];
+		iditht += filtering_kernel[X][Y][1];
+
+		idiths = idiths >> SHIFT16XYZ;
+		idiths = idiths ? idiths -1 : idiths;
+
+
+		iditht = iditht >> SHIFT16XYZ;
+		iditht = iditht ? iditht -1 : iditht;
+
+
+		*pdest++ = *(pbase + idiths + iditht * cachewidth);
+		s += sstep;
+		t += tstep;
+	} while (--spancount > 0);
+
+	return pdest;
+}
+
+/*
+=============
+D_DrawSpansPow2
+=============
+*/
+void
+D_DrawSpansPow2 (espan_t *pspan)
+{
+	int 	spancount;
+	pixel_t	*pbase;
+	int	snext, tnext;
+	float	spancountminus1;
+	float	sdivzpow2stepu, tdivzpow2stepu, zipow2stepu;
+	int	texture_filtering;
+	int	spanstep_shift, spanstep_value;
+
+	spanstep_shift = D_DrawSpanGetStep(d_zistepu, d_zistepv);
+	spanstep_value = (1 << spanstep_shift);
 
 	pbase = (unsigned char *)cacheblock;
 
-	sdivz8stepu = d_sdivzstepu * 8;
-	tdivz8stepu = d_tdivzstepu * 8;
-	zi8stepu = d_zistepu * 8;
+	texture_filtering = (int)sw_texture_filtering->value;
+	sdivzpow2stepu = d_sdivzstepu * spanstep_value;
+	tdivzpow2stepu = d_tdivzstepu * spanstep_value;
+	zipow2stepu = d_zistepu * spanstep_value;
 
 	do
 	{
 		pixel_t	*pdest;
-		int		count, s, t;
+		int	count, s, t;
 		float	sdivz, tdivz, zi, z, du, dv;
 
-		pdest = d_viewbuffer + (r_screenwidth * pspan->v) + pspan->u;
+		pdest = d_viewbuffer + (vid.width * pspan->v) + pspan->u;
 
 		count = pspan->count;
 
@@ -452,9 +644,14 @@ void D_DrawSpans16 (espan_t *pspan)
 
 		do
 		{
+			int	sstep, tstep;
+
+			sstep = 0;	// keep compiler happy
+			tstep = 0;	// ditto
+
 			// calculate s and t at the far end of the span
-			if (count >= 8)
-				spancount = 8;
+			if (count >= spanstep_value)
+				spancount = spanstep_value;
 			else
 				spancount = count;
 
@@ -464,27 +661,29 @@ void D_DrawSpans16 (espan_t *pspan)
 			{
 				// calculate s/z, t/z, zi->fixed s and t at far end of span,
 				// calculate s and t steps across span by shifting
-				sdivz += sdivz8stepu;
-				tdivz += tdivz8stepu;
-				zi += zi8stepu;
+				sdivz += sdivzpow2stepu;
+				tdivz += tdivzpow2stepu;
+				zi += zipow2stepu;
 				z = (float)SHIFT16XYZ_MULT / zi;	// prescale to 16.16 fixed-point
 
 				snext = (int)(sdivz * z) + sadjust;
 				if (snext > bbextents)
 					snext = bbextents;
-				else if (snext < 8)
-					snext = 8;	// prevent round-off error on <0 steps from
-							//  from causing overstepping & running off the
-							//  edge of the texture
+				else if (snext < spanstep_value)
+					// prevent round-off error on <0 steps from
+					//  from causing overstepping & running off the
+					//  edge of the texture
+					snext = spanstep_value;
 
 				tnext = (int)(tdivz * z) + tadjust;
 				if (tnext > bbextentt)
 					tnext = bbextentt;
-				else if (tnext < 8)
-					tnext = 8;	// guard against round-off error on <0 steps
+				else if (tnext < spanstep_value)
+					// guard against round-off error on <0 steps
+					tnext = spanstep_value;
 
-				sstep = (snext - s) >> 3;
-				tstep = (tnext - t) >> 3;
+				sstep = (snext - s) >> spanstep_shift;
+				tstep = (tnext - t) >> spanstep_shift;
 			}
 			else
 			{
@@ -500,16 +699,18 @@ void D_DrawSpans16 (espan_t *pspan)
 				snext = (int)(sdivz * z) + sadjust;
 				if (snext > bbextents)
 					snext = bbextents;
-				else if (snext < 8)
-					snext = 8;	// prevent round-off error on <0 steps from
-							//  from causing overstepping & running off the
-							//  edge of the texture
+				else if (snext < spanstep_value)
+					// prevent round-off error on <0 steps from
+					//  from causing overstepping & running off the
+					//  edge of the texture
+					snext = spanstep_value;
 
 				tnext = (int)(tdivz * z) + tadjust;
 				if (tnext > bbextentt)
 					tnext = bbextentt;
-				else if (tnext < 8)
-					tnext = 8;	// guard against round-off error on <0 steps
+				else if (tnext < spanstep_value)
+					// guard against round-off error on <0 steps
+					tnext = spanstep_value;
 
 				if (spancount > 1)
 				{
@@ -519,46 +720,18 @@ void D_DrawSpans16 (espan_t *pspan)
 			}
 
 			// Drawing phrase
-			if (sw_texture_filtering->value == 0.0f)
+			if (texture_filtering == 0)
 			{
-				do
-				{
-					*pdest++ = *(pbase + (s >> SHIFT16XYZ) + (t >> SHIFT16XYZ) * cachewidth);
-					s += sstep;
-					t += tstep;
-				} while (--spancount > 0);
-
-				s = snext;
-				t = tnext;
+				pdest = D_DrawSpan(pdest, pbase, s, t, sstep, tstep,
+						   spancount);
 			}
-			else if (sw_texture_filtering->value == 1.0f)
+			else
 			{
-				do
-				{
-					int idiths = s;
-					int iditht = t;
-
-					int X = (pspan->u + spancount) & 1;
-					int Y = (pspan->v)&1;
-
-					//Using the kernel
-					idiths += filtering_kernel[X][Y][0];
-					iditht += filtering_kernel[X][Y][1];
-
-					idiths = idiths >> SHIFT16XYZ;
-					idiths = idiths ? idiths -1 : idiths;
-
-
-					iditht = iditht >> SHIFT16XYZ;
-					iditht = iditht ? iditht -1 : iditht;
-
-
-					*pdest++ = *(pbase + idiths + iditht * cachewidth);
-					s += sstep;
-					t += tstep;
-				} while (--spancount > 0);
+				pdest = D_DrawSpanFiltered(pdest, pbase, s, t, sstep, tstep,
+						   spancount, pspan);
 			}
-
+			s = snext;
+			t = tnext;
 		} while (count > 0);
 
 	} while ((pspan = pspan->pnext) != NULL);
@@ -569,23 +742,26 @@ void D_DrawSpans16 (espan_t *pspan)
 D_DrawZSpans
 =============
 */
-void D_DrawZSpans (espan_t *pspan)
+void
+D_DrawZSpans (espan_t *pspan)
 {
-	int izistep;
+	zvalue_t	izistep;
+	int		safe_step;
 
 	// FIXME: check for clamping/range problems
 	// we count on FP exceptions being turned off to avoid range problems
 	izistep = (int)(d_zistepu * 0x8000 * (float)SHIFT16XYZ_MULT);
+	safe_step = D_DrawZSpanGetStepValue(izistep);
 
 	do
 	{
 		int		count;
-		int		izi;
+		zvalue_t	izi;
 		zvalue_t	*pdest;
 		float		zi;
 		float		du, dv;
 
-		pdest = d_pzbuffer + (d_zwidth * pspan->v) + pspan->u;
+		pdest = d_pzbuffer + (vid.width * pspan->v) + pspan->u;
 
 		count = pspan->count;
 
@@ -597,11 +773,29 @@ void D_DrawZSpans (espan_t *pspan)
 		// we count on FP exceptions being turned off to avoid range problems
 		izi = (int)(zi * 0x8000 * (float)SHIFT16XYZ_MULT);
 
-		while (count > 0)
+		if (safe_step > 1)
 		{
-			*pdest++ = izi >> SHIFT16XYZ;
-			izi += izistep;
-			count--;
+			const zvalue_t *tdest_max = pdest + count;
+			do
+			{
+				int	step;
+				zvalue_t izi_shifted = izi >> SHIFT16XYZ;
+
+				for(step = 0; (step < safe_step) && (pdest < tdest_max); step++)
+				{
+					*pdest++ = izi_shifted;
+				}
+				izi += (izistep * safe_step);
+			} while (pdest < tdest_max);
+		}
+		else
+		{
+			while (count > 0)
+			{
+				*pdest++ = izi >> SHIFT16XYZ;
+				izi += izistep;
+				count--;
+			}
 		}
 	} while ((pspan = pspan->pnext) != NULL);
 }
