@@ -32,7 +32,7 @@
 #include "shared.h"
 #include "crc.h"
 
-#define YQ2VERSION "7.21"
+#define YQ2VERSION "7.40"
 #define BASEDIRNAME "baseq2"
 
 #ifndef YQ2OSTYPE
@@ -635,8 +635,6 @@ void Pmove(pmove_t *pmove);
 
 #define SFF_INPACK 0x20
 
-extern qboolean file_from_pak;
-
 typedef int fileHandle_t;
 
 typedef enum
@@ -676,6 +674,8 @@ void FS_BuildGameSpecificSearchPath(char *dir);
 char *FS_Gamedir(void);
 char *FS_NextPath(char *prevpath);
 int FS_LoadFile(char *path, void **buffer);
+qboolean FS_FileInGamedir(const char *file);
+qboolean FS_AddPAKFromGamedir(const char *pak);
 const char* FS_GetNextRawPath(const char* lastRawPath);
 
 /* a null buffer will just return the file length without loading */
@@ -720,6 +720,9 @@ extern cvar_t *dedicated;
 extern cvar_t *host_speeds;
 extern cvar_t *log_stats;
 
+/* External entity files. */
+extern cvar_t *sv_entfile;
+
 /* Hack for portable client */
 extern qboolean is_portable;
 
@@ -778,6 +781,9 @@ void Sys_Error(char *error, ...);
 void Sys_Quit(void);
 void Sys_Init(void);
 char *Sys_GetHomeDir(void);
+void Sys_Remove(const char *path);
+int Sys_Rename(const char *from, const char *to);
+void Sys_RemoveDir(const char *path);
 long long Sys_Microseconds(void);
 void Sys_Nanosleep(int);
 void *Sys_GetProcAddress(void *handle, const char *sym);
