@@ -404,8 +404,8 @@ ExitLevel(void)
 		}
 	}
 
+	debristhisframe = 0;
 	gibsthisframe = 0;
-	lastgibframe = 0;
 }
 
 /*
@@ -420,6 +420,9 @@ G_RunFrame(void)
 	level.framenum++;
 	level.time = level.framenum * FRAMETIME;
 
+	gibsthisframe = 0;
+	debristhisframe = 0;
+
 	/* choose a client for monsters to target this frame */
 	AI_SetSightClient();
 
